# -*- coding: utf-8 -*-

############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 AKFish <akfish@gmail.com>                                     #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import github.GithubObject


class HookResponse(github.GithubObject.NonCompletableGithubObject):
    """
    This class represents HookResponses
    """

    def __repr__(self):
        return self.get__repr__({"status": self._status.value})

    @property
    def code(self):
        """
        :type: integer
        """
        return self._code.value

    @property
    def message(self):
        """
        :type: string
        """
        return self._message.value

    @property
    def status(self):
        """
        :type: string
        """
        return self._status.value

    def _initAttributes(self):
        self._code = github.GithubObject.NotSet
        self._message = github.GithubObject.NotSet
        self._status = github.GithubObject.NotSet

    def _useAttributes(self, attributes):
        if "code" in attributes:  # pragma no branch
            self._code = self._makeIntAttribute(attributes["code"])
        if "message" in attributes:  # pragma no branch
            self._message = self._makeStringAttribute(attributes["message"])
        if "status" in attributes:  # pragma no branch
            self._status = self._makeStringAttribute(attributes["status"])



import re
from urllib.parse import quote_plus

from requests.utils import dict_from_cookiejar

from sickchill import logger
from sickchill.helper.common import convert_size, try_int
from sickchill.oldbeard import tvcache
from sickchill.oldbeard.bs4_parser import BS4Parser
from sickchill.providers.torrent.TorrentProvider import TorrentProvider


class Provider(TorrentProvider):
    def __init__(self):

        super().__init__("HDTorrents.it")

        self.username = None
        self.password = None
        self.ratio = None
        self.minseed = 0
        self.minleech = 0
        self.freeleech = None

        self.urls = {
            "base_url": "http://hdtorrents.it",
            "login": "http://hdtorrents.it/takelogin.php",
            "search": "http://hdtorrents.it/browse.php?search=%s",
            "rss": "http://hdtorrents.it/browse.php?search=%s",
            "home": "http://hdtorrents.it/%s",
        }

        self.url = self.urls["base_url"]

        self.proper_strings = ["PROPER", "REPACK"]

        self.cache = tvcache.TVCache(self, min_time=30)  # only poll HDTorrents every 30 minutes ma

    def _check_auth(self):

        if not self.username or not self.password:
            logger.warning("Invalid username or password. Check your settings")

        return True

    def login(self):
        if any(dict_from_cookiejar(self.session.cookies).values()):
            return True

        login_params = {"username": self.username, "password": self.password, "submit": "Accedi!"}

        response = self.get_url(self.urls["login"], post_data=login_params, timeout=30)
        if not response:
            logger.warning("Unable to connect to provider")
            return False

        if re.search("Lei non e registrato in sistema.", response):
            logger.warning("Invalid username or password. Check your settings")
            return False

        return True

    def search(self, search_strings, age=0, ep_obj=None):
        results = []
        if not self.login():
            return results

        for mode in search_strings:
            items = []
            logger.debug("Search Mode: {}".format(mode))
            for search_string in {*search_strings[mode]}:
                if mode != "RSS":
                    search_url = self.urls["search"] % quote_plus(search_string)
                    logger.debug("Search string: {}".format(search_string))
                else:
                    search_url = self.urls["rss"]

                if self.freeleech:
                    search_url = search_url.replace("active=1", "active=5")

                logger.debug("Search URL: {}".format(search_url))

                data = self.get_url(search_url)
                if not data or "Error" in data:
                    logger.debug("No data returned from provider")
                    continue

                if data.find("Non abbiamo trovato nulla") != -1:
                    logger.debug("Data returned from provider does not contain any torrents")
                    continue

                # Search result page contains some invalid html that prevents html parser from returning all data.
                # We cut everything before the table that contains the data we are interested in thus eliminating
                # the invalid html portions
                try:
                    index = data.lower().index('<tbody id="highlighted"')
                except ValueError:
                    logger.debug("Could not find table of torrents highlighted")
                    continue

                # data = urllib.unquote(data[index:]).replace('\t', '')
                data = data[index:]

                with BS4Parser(data, "html5lib") as html:
                    if not html:
                        logger.debug("No html data parsed from provider")
                        continue

                    torrent_rows = []
                    torrent_table = html.find("table", class_="highlighted")
                    if torrent_table:
                        torrent_rows = torrent_table.find_all("tr")

                    if not torrent_rows:
                        logger.debug("Could not find results in returned data")
                        continue

                    # Cat., Active, Filename, Dl, Wl, Added, Size, Uploader, S, L, C
                    labels = [label.a.get_text(strip=True) if label.a else label.get_text(strip=True) for label in torrent_rows[0].find_all("td")]

                    # Skip column headers
                    for result in torrent_rows[1:]:
                        try:
                            cells = result.find_all("td")[: len(labels)]
                            if len(cells) < len(labels):
                                continue

                            title = cells[labels.index(1)].a.index(0).get_text(strip=True)
                            seeders = try_int(cells[labels.index(5)].a.index(0).get_text(strip=True))
                            leechers = try_int(cells[labels.index(5)].a.index(1).get_text(strip=True))
                            torrent_size = cells[labels.index(4)].get_text()

                            size = convert_size(torrent_size) or -1
                            download_url = self.url + "/" + cells[labels.index(1)].a.index(0)["href"]
                            # title = cells[labels.index('Filename')].a.get_text(strip=True)
                            # seeders = try_int(cells[labels.index('S')].get_text(strip=True))
                            # leechers = try_int(cells[labels.index('L')].get_text(strip=True))
                            # torrent_size = cells[labels.index('Size')].get_text()

                            # size = convert_size(torrent_size) or -1
                            # download_url = self.url + '/' + cells[labels.index('Dl')].a['href']
                        except (AttributeError, TypeError, KeyError, ValueError, IndexError):
                            continue

                        if not all([title, download_url]):
                            continue

                        # Filter unseeded torrent
                        if seeders < self.minseed or leechers < self.minleech:
                            if mode != "RSS":
                                logger.debug(
                                    "Discarding torrent because it doesn't meet the minimum seeders or leechers: {0} (S:{1} L:{2})".format(
                                        title, seeders, leechers
                                    )
                                )
                            continue

                        item = title, download_url, size, seeders, leechers
                        if mode != "RSS":
                            logger.debug("Found result: {0} with {1} seeders and {2} leechers".format(title, seeders, leechers))

                        items.append(item)

            # For each search mode sort all the items by seeders if available
            items.sort(key=lambda tup: tup[3], reverse=True)

            results += items

        return results



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: generated by @lolobosse script
'''
LOCALE = [
    ["poco fa", "tra poco"],
    ["%s secondi fa", "%s secondi da ora"],
    ["un minuto fa", "un minuto da ora"],
    ["%s minuti fa", "%s minuti da ora"],
    ["un'ora fa", "un'ora da ora"],
    ["%s ore fa", "%s ore da ora"],
    ["un giorno fa", "un giorno da ora"],
    ["%s giorni fa", "%s giorni da ora"],
    ["una settimana fa", "una settimana da ora"],
    ["%s settimane fa", "%s settimane da ora"],
    ["un mese fa", "un mese da ora"],
    ["%s mesi fa", "%s mesi da ora"],
    ["un anno fa", "un anno da ora"],
    ["%s anni fa", "%s anni da ora"]
]
        


import gettext
import os
import sys


def setup_lib_path(additional=None):
    lib_path = os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), "lib3"))
    if lib_path not in sys.path:
        sys.path.insert(1, lib_path)
    if additional and additional not in sys.path:
        sys.path.insert(1, additional)


def sickchill_dir():
    return os.path.abspath(os.path.dirname(__file__))


def locale_dir():
    return os.path.abspath(os.path.join(os.path.dirname(__file__), "locale"))


def setup_gettext(language=None):
    languages = [language] if language else None
    if not [key for key in ("LANGUAGE", "LC_ALL", "LC_MESSAGES", "LANG") if os.environ.get(key)]:
        os.environ["LC_MESSAGES"] = "en_US.UTF-8"
    print(locale_dir())
    gt = gettext.translation("messages", locale_dir(), languages=languages, fallback=True)
    gt.install(names=["ngettext"])


def check_installed():
    try:
        from importlib.metadata import Distribution, PackageNotFoundError
    except ImportError:
        from importlib_metadata import Distribution, PackageNotFoundError

    try:
        Distribution.from_name("sickchill")
    except PackageNotFoundError:
        return False
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable-all
from __future__ import print_function

import os
import re
import sys
import codecs
import shutil
import argparse
from textwrap import dedent

from chardet import detect
from pysrt import SubRipFile, SubRipTime, VERSION_STRING


def underline(string):
    return "\033[4m%s\033[0m" % string


class TimeAwareArgumentParser(argparse.ArgumentParser):

    RE_TIME_REPRESENTATION = re.compile(r'^\-?(\d+[hms]{0,2}){1,4}$')

    def parse_args(self, args=None, namespace=None):
        time_index = -1
        for index, arg in enumerate(args):
            match = self.RE_TIME_REPRESENTATION.match(arg)
            if match:
                time_index = index
                break

        if time_index >= 0:
            args.insert(time_index, '--')

        return super(TimeAwareArgumentParser, self).parse_args(args, namespace)


class SubRipShifter(object):

    BACKUP_EXTENSION = '.bak'
    RE_TIME_STRING = re.compile(r'(\d+)([hms]{0,2})')
    UNIT_RATIOS = {
        'ms': 1,
        '': SubRipTime.SECONDS_RATIO,
        's': SubRipTime.SECONDS_RATIO,
        'm': SubRipTime.MINUTES_RATIO,
        'h': SubRipTime.HOURS_RATIO,
    }
    DESCRIPTION = dedent("""\
        Srt subtitle editor

        It can either shift, split or change the frame rate.
    """)
    TIMESTAMP_HELP = "A timestamp in the form: [-][Hh][Mm]S[s][MSms]"
    SHIFT_EPILOG = dedent("""\

        Examples:
            1 minute and 12 seconds foreward (in place):
                $ srt -i shift 1m12s movie.srt

            half a second foreward:
                $ srt shift 500ms movie.srt > othername.srt

            1 second and half backward:
                $ srt -i shift -1s500ms movie.srt

            3 seconds backward:
                $ srt -i shift -3 movie.srt
    """)
    RATE_EPILOG = dedent("""\

        Examples:
            Convert 23.9fps subtitles to 25fps:
                $ srt -i rate 23.9 25 movie.srt
    """)
    LIMITS_HELP = "Each parts duration in the form: [Hh][Mm]S[s][MSms]"
    SPLIT_EPILOG = dedent("""\

        Examples:
            For a movie in 2 parts with the first part 48 minutes and 18 seconds long:
                $ srt split 48m18s movie.srt
                => creates movie.1.srt and movie.2.srt

            For a movie in 3 parts of 20 minutes each:
                $ srt split 20m 20m movie.srt
                => creates movie.1.srt, movie.2.srt and movie.3.srt
    """)
    FRAME_RATE_HELP = "A frame rate in fps (commonly 23.9 or 25)"
    ENCODING_HELP = dedent("""\
        Change file encoding. Useful for players accepting only latin1 subtitles.
        List of supported encodings: http://docs.python.org/library/codecs.html#standard-encodings
    """)
    BREAK_EPILOG = dedent("""\
        Break lines longer than defined length
    """)
    LENGTH_HELP = "Maximum number of characters per line"

    def __init__(self):
        self.output_file_path = None

    def build_parser(self):
        parser = TimeAwareArgumentParser(description=self.DESCRIPTION, formatter_class=argparse.RawTextHelpFormatter)
        parser.add_argument('-i', '--in-place', action='store_true', dest='in_place',
            help="Edit file in-place, saving a backup as file.bak (do not works for the split command)")
        parser.add_argument('-e', '--output-encoding', metavar=underline('encoding'), action='store', dest='output_encoding',
            type=self.parse_encoding, help=self.ENCODING_HELP)
        parser.add_argument('-v', '--version', action='version', version='%%(prog)s %s' % VERSION_STRING)
        subparsers = parser.add_subparsers(title='commands')

        shift_parser = subparsers.add_parser('shift', help="Shift subtitles by specified time offset", epilog=self.SHIFT_EPILOG, formatter_class=argparse.RawTextHelpFormatter)
        shift_parser.add_argument('time_offset', action='store', metavar=underline('offset'),
            type=self.parse_time, help=self.TIMESTAMP_HELP)
        shift_parser.set_defaults(action=self.shift)

        rate_parser = subparsers.add_parser('rate', help="Convert subtitles from a frame rate to another", epilog=self.RATE_EPILOG, formatter_class=argparse.RawTextHelpFormatter)
        rate_parser.add_argument('initial', action='store', type=float, help=self.FRAME_RATE_HELP)
        rate_parser.add_argument('final', action='store', type=float, help=self.FRAME_RATE_HELP)
        rate_parser.set_defaults(action=self.rate)

        split_parser = subparsers.add_parser('split', help="Split a file in multiple parts", epilog=self.SPLIT_EPILOG, formatter_class=argparse.RawTextHelpFormatter)
        split_parser.add_argument('limits', action='store', nargs='+', type=self.parse_time, help=self.LIMITS_HELP)
        split_parser.set_defaults(action=self.split)

        break_parser = subparsers.add_parser('break', help="Break long lines", epilog=self.BREAK_EPILOG, formatter_class=argparse.RawTextHelpFormatter)
        break_parser.add_argument('length', action='store', type=int, help=self.LENGTH_HELP)
        break_parser.set_defaults(action=self.break_lines)

        parser.add_argument('file', action='store')

        return parser

    def run(self, args):
        self.arguments = self.build_parser().parse_args(args)

        if os.path.isfile(self.arguments.file):
            if self.arguments.in_place:
                self.create_backup()
            self.arguments.action()

        else:
            print('No such file', self.arguments.file)

    def parse_time(self, time_string):
        negative = time_string.startswith('-')
        if negative:
            time_string = time_string[1:]
        ordinal = sum(int(value) * self.UNIT_RATIOS[unit] for value, unit
                        in self.RE_TIME_STRING.findall(time_string))
        return -ordinal if negative else ordinal

    def parse_encoding(self, encoding_name):
        try:
            codecs.lookup(encoding_name)
        except LookupError as error:
            raise argparse.ArgumentTypeError(error.message)
        return encoding_name

    def shift(self):
        self.input_file.shift(milliseconds=self.arguments.time_offset)
        self.input_file.write_into(self.output_file)

    def rate(self):
        ratio = self.arguments.final / self.arguments.initial
        self.input_file.shift(ratio=ratio)
        self.input_file.write_into(self.output_file)

    def split(self):
        limits = [0] + self.arguments.limits + [self.input_file[-1].end.ordinal + 1]
        base_name, extension = os.path.splitext(self.arguments.file)
        for index, (start, end) in enumerate(zip(limits[:-1], limits[1:])):
            file_name = '%s.%s%s' % (base_name, index + 1, extension)
            part_file = self.input_file.slice(ends_after=start, starts_before=end)
            part_file.shift(milliseconds=-start)
            part_file.clean_indexes()
            part_file.save(path=file_name, encoding=self.output_encoding)

    def create_backup(self):
        backup_file = self.arguments.file + self.BACKUP_EXTENSION
        if not os.path.exists(backup_file):
            shutil.copy2(self.arguments.file, backup_file)
        self.output_file_path = self.arguments.file
        self.arguments.file = backup_file

    def break_lines(self):
        split_re = re.compile(r'(.{,%i})(?:\s+|$)' % self.arguments.length)
        for item in self.input_file:
            item.text = '\n'.join(split_re.split(item.text)[1::2])
        self.input_file.write_into(self.output_file)

    @property
    def output_encoding(self):
        return self.arguments.output_encoding or self.input_file.encoding

    @property
    def input_file(self):
        if not hasattr(self, '_source_file'):
            with open(self.arguments.file, 'rb') as f:
                content = f.read()
                encoding = detect(content).get('encoding')
                encoding = self.normalize_encoding(encoding)

            self._source_file = SubRipFile.open(self.arguments.file,
                encoding=encoding, error_handling=SubRipFile.ERROR_LOG)
        return self._source_file

    @property
    def output_file(self):
        if not hasattr(self, '_output_file'):
            if self.output_file_path:
                self._output_file = codecs.open(self.output_file_path, 'w+', encoding=self.output_encoding)
            else:
                self._output_file = sys.stdout
        return self._output_file

    def normalize_encoding(self, encoding):
        return encoding.lower().replace('-', '_')


def main():
    SubRipShifter().run(sys.argv[1:])

if __name__ == '__main__':
    main()



import validators

from sickchill import logger
from sickchill.helper.common import convert_size, try_int
from sickchill.oldbeard import tvcache
from sickchill.providers.torrent.TorrentProvider import TorrentProvider


class Provider(TorrentProvider):
    def __init__(self):

        super().__init__("Nyaa")

        self.public = True
        self.supports_absolute_numbering = True
        self.anime_only = True

        self.url = "https://nyaa.si"
        self.custom_url = None

        self.minseed = 0
        self.minleech = 0
        self.confirmed = False

        self.cache = tvcache.TVCache(self, min_time=20)  # only poll Nyaa every 20 minutes max

    def search(self, search_strings, age=0, ep_obj=None):
        results = []
        if self.show and not self.show.is_anime:
            return results

        if self.custom_url:
            if not validators.url(self.custom_url):
                logger.warning("Invalid custom url: {0}".format(self.custom_url))
                return results

        for mode in search_strings:
            items = []
            logger.debug(_("Search Mode: {mode}".format(mode=mode)))
            for search_string in {*search_strings[mode]}:
                if mode != "RSS":
                    logger.debug(_("Search String: {search_string}".format(search_string=search_string)))

                search_params = {
                    "page": "rss",
                    "c": "1_0",  # Category: All anime
                    "s": "id",  # Sort by: 'id'=Date / 'size' / 'name' / 'seeders' / 'leechers' / 'downloads'
                    "o": "desc",  # Sort direction: asc / desc
                    "f": ("0", "2")[self.confirmed],  # Quality filter: 0 = None / 1 = No Remakes / 2 = Trusted Only
                }
                if mode != "RSS":
                    search_params["q"] = search_string

                results = []
                data = self.cache.get_rss_feed(self.custom_url or self.url, params=search_params)["entries"]
                if not data:
                    logger.debug("Data returned from provider does not contain any torrents")
                    continue

                for curItem in data:
                    try:
                        title = curItem["title"]
                        download_url = curItem["link"]
                        if not all([title, download_url]):
                            continue

                        seeders = try_int(curItem["nyaa_seeders"])
                        leechers = try_int(curItem["nyaa_leechers"])
                        torrent_size = curItem["nyaa_size"]
                        info_hash = curItem["nyaa_infohash"]

                        if seeders < self.minseed or leechers < self.minleech:
                            if mode != "RSS":
                                logger.debug(
                                    "Discarding torrent because it doesn't meet the"
                                    " minimum seeders or leechers: {0} (S:{1} L:{2})".format(title, seeders, leechers)
                                )
                            continue

                        size = convert_size(torrent_size, units=["BYTES", "KIB", "MIB", "GIB", "TIB", "PIB"]) or -1
                        result = {"title": title, "link": download_url, "size": size, "seeders": seeders, "leechers": leechers, "hash": info_hash}
                        if mode != "RSS":
                            logger.debug(
                                _(
                                    "Found result: {title} with {seeders} seeders and {leechers} leechers".format(
                                        title=title, seeders=seeders, leechers=leechers
                                    )
                                )
                            )

                        items.append(result)
                    except Exception:
                        continue

            # For each search mode sort all the items by seeders
            items.sort(key=lambda d: d.get("seeders", 0), reverse=True)
            results += items

        return results



#!/usr/bin/python
# -- Content-Encoding: UTF-8 --
"""
The configuration module.

:copyright: Copyright 2020, Thomas Calmant
:license: Apache License 2.0
:version: 0.4.1

..

    Copyright 2020 Thomas Calmant

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import sys

# ------------------------------------------------------------------------------

# Module version
__version_info__ = (0, 4, 1)
__version__ = ".".join(str(x) for x in __version_info__)

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------


class LocalClasses(dict):
    """
    Associates local classes with their names (used in the jsonclass module)
    """
    def add(self, cls, name=None):
        """
        Stores a local class

        :param cls: A class
        :param name: Custom name used in the __jsonclass__ attribute
        """
        self[name or cls.__name__] = cls

# ------------------------------------------------------------------------------


class Config(object):
    """
    This is pretty much used exclusively for the 'jsonclass'
    functionality... set use_jsonclass to False to turn it off.
    You can change serialize_method and ignore_attribute, or use
    the local_classes.add(class) to include "local" classes.
    """
    def __init__(self, version=2.0, content_type="application/json-rpc",
                 user_agent=None, use_jsonclass=True,
                 serialize_method='_serialize',
                 ignore_attribute='_ignore',
                 serialize_handlers=None):
        """
        Sets up a configuration of JSONRPClib

        :param version: JSON-RPC specification version
        :param content_type: HTTP content type header value
        :param user_agent: The HTTP request user agent
        :param use_jsonclass: Allow bean marshalling
        :param serialize_method: A string that references the method on a
                                 custom class object which is responsible for
                                 returning a tuple of the arguments and a dict
                                 of attributes.
        :param ignore_attribute: A string that references the attribute on a
                                 custom class object which holds strings and/or
                                 references of the attributes the class
                                 translator should ignore.
        :param serialize_handlers: A dictionary of dump handler functions by
                                   type for additional type support and for
                                   overriding dump of built-in types in utils
        """
        # JSON-RPC specification
        self.version = version

        # Change to False to keep __jsonclass__ entries raw.
        self.use_jsonclass = use_jsonclass

        # it SHOULD be 'application/json-rpc'
        # but MAY be 'application/json' or 'application/jsonrequest'
        self.content_type = content_type

        # Default user agent
        if user_agent is None:
            user_agent = 'jsonrpclib/{0} (Python {1})'.format(
                __version__,
                '.'.join(str(ver) for ver in sys.version_info[0:3]))
        self.user_agent = user_agent

        # The list of classes to use for jsonclass translation.
        self.classes = LocalClasses()

        # The serialize_method should be a string that references the
        # method on a custom class object which is responsible for
        # returning a tuple of the constructor arguments and a dict of
        # attributes.
        self.serialize_method = serialize_method

        # The ignore attribute should be a string that references the
        # attribute on a custom class object which holds strings and / or
        # references of the attributes the class translator should ignore.
        self.ignore_attribute = ignore_attribute

        # The list of serialize handler functions for jsonclass dump.
        # Used for handling additional types and overriding built-in types.
        # Functions are expected to have the same parameters as jsonclass dump
        # (possibility to call standard jsonclass dump function within).
        self.serialize_handlers = serialize_handlers or {}

    def copy(self):
        """
        Returns a shallow copy of this configuration bean

        :return: A shallow copy of this configuration
        """
        new_config = Config(self.version, self.content_type, self.user_agent,
                            self.use_jsonclass, self.serialize_method,
                            self.ignore_attribute, None)
        new_config.classes = self.classes.copy()
        new_config.serialize_handlers = self.serialize_handlers.copy()
        return new_config

# Default configuration
DEFAULT = Config()



"""
Test coming episodes
"""


import unittest

from sickchill.show.ComingEpisodes import ComingEpisodes


class ComingEpisodesTests(unittest.TestCase):
    """
    Test comping episodes
    """

    def test_get_categories(self):
        """
        Tests if get_categories returns the valid format and the right values
        """
        categories_list = [
            None,
            [],
            ["A", "B"],
            ["A", "B"],
            "",
            "A|B",
            "A|B",
        ]
        results_list = [[], [], ["A", "B"], ["A", "B"], [], ["A", "B"], ["A", "B"]]

        assert len(categories_list) == len(results_list), "Number of parameters ({0:d}) and results ({1:d}) does not match".format(
            len(categories_list), len(results_list))

        for (index, categories) in enumerate(categories_list):
            assert ComingEpisodes._get_categories(categories) == results_list[index]

    def test_get_categories_map(self):
        """
        Tests if get_categories_map returns the valid format and the right values
        """
        categories_list = [None, [], ["A", "B"], ["A", "B"]]
        results_list = [{}, {}, {"A": [], "B": []}, {"A": [], "B": []}]

        assert len(categories_list) == len(results_list), "Number of parameters ({0:d}) and results ({1:d}) does not match".format(
            len(categories_list), len(results_list))

        for (index, categories) in enumerate(categories_list):
            assert ComingEpisodes._get_categories_map(categories) == results_list[index]

    def test_get_sort(self):
        """
        Tests if get_sort returns the right sort of coming episode
        """
        test_cases = {
            None: "date",
            "": "date",
            "wrong": "date",
            "date": "date",
            "Date": "date",
            "network": "network",
            "NetWork": "network",
            "show": "show",
            "Show": "show",
        }

        unicode_test_cases = {
            "": "date",
            "wrong": "date",
            "date": "date",
            "Date": "date",
            "network": "network",
            "NetWork": "network",
            "show": "show",
            "Show": "show",
        }

        for tests in test_cases, unicode_test_cases:
            for (sort, result) in tests.items():
                assert ComingEpisodes._get_sort(sort) == result


if __name__ == "__main__":
    print("=====> Testing {0}".format(__file__))

    SUITE = unittest.TestLoader().loadTestsFromTestCase(ComingEpisodesTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)



"""
Test notifiers
"""


import unittest

from sickchill.oldbeard import db
from sickchill.oldbeard.notifiers.emailnotify import Notifier as EmailNotifier
from sickchill.oldbeard.notifiers.prowl import Notifier as ProwlNotifier
from sickchill.tv import TVEpisode, TVShow
from sickchill.views.home import Home
from tests import conftest


# noinspection PyProtectedMember
class NotifierTests(conftest.SickChillTestDBCase):
    """
    Test notifiers
    """

    @classmethod
    def setUpClass(cls):
        num_legacy_shows = 3
        num_shows = 3
        num_episodes_per_show = 5
        cls.mydb = db.DBConnection()
        cls.legacy_shows = []
        cls.shows = []

        # Per-show-notifications were originally added for email notifications only.  To add
        # this feature to other notifiers, it was necessary to alter the way text is stored in
        # one of the DB columns.  Therefore, to test properly, we must create some shows that
        # store emails in the old method (legacy method) and then other shows that will use
        # the new method.
        for show_counter in range(100, 100 + num_legacy_shows):
            show = TVShow(1, show_counter)
            show.name = "Show " + str(show_counter)
            show.episodes = []
            for episode_counter in range(0, num_episodes_per_show):
                episode = TVEpisode(show, conftest.SEASON, episode_counter)
                episode.name = "Episode " + str(episode_counter + 1)
                episode.quality = "SDTV"
                show.episodes.append(episode)
            show.saveToDB()
            cls.legacy_shows.append(show)

        for show_counter in range(200, 200 + num_shows):
            show = TVShow(1, show_counter)
            show.name = "Show " + str(show_counter)
            show.episodes = []
            for episode_counter in range(0, num_episodes_per_show):
                episode = TVEpisode(show, conftest.SEASON, episode_counter)
                episode.name = "Episode " + str(episode_counter + 1)
                episode.quality = "SDTV"
                show.episodes.append(episode)
            show.saveToDB()
            cls.shows.append(show)

    def setUp(self):
        """
        Set up tests
        """
        self._debug_spew("\n\r")

    @unittest.skip("Not yet implemented")
    def test_boxcar(self):
        """
        Test boxcar notifications
        """
        pass

    @unittest.skip("Cannot call directly without a request")
    def test_email(self):
        """
        Test email notifications
        """
        email_notifier = EmailNotifier()

        # Per-show-email notifications were added early on and utilized a different format than the other notifiers.
        # Therefore, to test properly (and ensure backwards compatibility), this routine will test shows that use
        # both the old and the new storage methodology
        legacy_test_emails = "email-1@address.com,email2@address.org,email_3@address.tv"
        test_emails = "email-4@address.com,email5@address.org,email_6@address.tv"

        for show in self.legacy_shows:
            showid = self._get_showid_by_showname(show.show_name)
            self.mydb.action("UPDATE tv_shows SET notify_list = ? WHERE show_id = ?", [legacy_test_emails, showid])

        for show in self.shows:
            showid = self._get_showid_by_showname(show.show_name)
            Home.saveShowNotifyList(show=showid, emails=test_emails)

        # Now, iterate through all shows using the email list generation routines that are used in the notifier proper
        shows = self.legacy_shows + self.shows
        for show in shows:
            for episode in show.episodes:
                ep_name = episode._format_pattern("%SN - %Sx%0E - %EN - ") + episode.quality
                show_name = email_notifier._parseEp(ep_name)
                recipients = email_notifier._generate_recipients(show_name)
                self._debug_spew("- Email Notifications for " + show.name + " (episode: " + episode.name + ") will be sent to:")
                for email in recipients:
                    self._debug_spew("-- " + email.strip())
                self._debug_spew("\n\r")

        return True

    @unittest.skip("Not yet implemented")
    def test_emby(self):
        """
        Test emby notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_freemobile(self):
        """
        Test freemobile notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_growl(self):
        """
        Test growl notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_kodi(self):
        """
        Test kodi notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_libnotify(self):
        """
        Test libnotify notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_nma(self):
        """
        Test nma notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_nmj(self):
        """
        Test nmj notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_nmjv2(self):
        """
        Test nmjv2 notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_plex(self):
        """
        Test plex notifications
        """
        pass

    @unittest.skip("Cannot call directly without a request")
    def test_prowl(self):
        """
        Test prowl notifications
        """
        prowl_notifier = ProwlNotifier()

        # Prowl per-show-notifications only utilize the new methodology for storage; therefore, the list of legacy_shows
        # will not be altered (to preserve backwards compatibility testing)
        test_prowl_apis = "11111111111111111111,22222222222222222222"

        for show in self.shows:
            showid = self._get_showid_by_showname(show.show_name)
            Home.saveShowNotifyList(show=showid, prowlAPIs=test_prowl_apis)

        # Now, iterate through all shows using the Prowl API generation routines that are used in the notifier proper
        for show in self.shows:
            for episode in show.episodes:
                ep_name = episode._format_pattern("%SN - %Sx%0E - %EN - ") + episode.quality
                show_name = prowl_notifier._parse_episode(ep_name)
                recipients = prowl_notifier._generate_recipients(show_name)
                self._debug_spew("- Prowl Notifications for " + show.name + " (episode: " + episode.name + ") will be sent to:")
                for api in recipients:
                    self._debug_spew("-- " + api.strip())
                self._debug_spew("\n\r")

        return True

    @unittest.skip("Not yet implemented")
    def test_pushalot(self):
        """
        Test pushalot notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_pushbullet(self):
        """
        Test pushbullet notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_pushover(self):
        """
        Test pushover notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_pytivo(self):
        """
        Test pytivo notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_synoindex(self):
        """
        Test synoindex notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_synologynotifier(self):
        """
        Test synologynotifier notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_trakt(self):
        """
        Test trakt notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_tweet(self):
        """
        Test tweet notifications
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_twilio(self):
        """
        Test twilio notifications
        """
        pass

    @staticmethod
    def _debug_spew(text):
        """
        Spew text notifications

        :param text: to spew
        :return:
        """
        if __name__ == "__main__" and text is not None:
            print(text)

    def _get_showid_by_showname(self, showname):
        """
        Get show ID by show name

        :param showname:
        :return:
        """
        if showname is not None:
            rows = self.mydb.select("SELECT show_id FROM tv_shows WHERE show_name = ?", [showname])
            if len(rows) == 1:
                return rows[0]["show_id"]
        return -1


if __name__ == "__main__":
    print("==================")
    print("STARTING - NOTIFIER TESTS")
    print("==================")
    print("######################################################################")

    SUITE = unittest.TestLoader().loadTestsFromTestCase(NotifierTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)



import uuid

from beekeeper import API, VariableHandler, render_variables

from kodipydent.rpctohive import Hive

base_hive = {
    'root': 'http://{hostname}:{port}',
    'variables': {
        'hostname': {
            'type': 'url_replacement'
        },
        'port': {
            'type': 'url_replacement'
        },
        'username': {
            'type': 'http_basic_auth',
            'optional': True,
        },
        'password': {
            'type': 'http_basic_auth',
            'optional': True
        },
        'jsonrpc': {
            'value': '2.0',
            'type': 'json_rpc'
        },
        'request_id': {
            'type': 'json_rpc'
        }
    },
    'endpoints': {
        'rpc': {
            'path': '/jsonrpc',
            'methods': ['POST']
        }
    },
    'objects': {
        'API': {
            'actions': {
                'get': {
                    'endpoint': 'rpc',
                    'method': 'POST',
                    'traverse': ['result'],
                    'variables': {
                        'method': {
                            'value': 'JSONRPC.Introspect',
                            'type': 'json_rpc'
                        }
                    }
                }
            }
        }
    }
}

@VariableHandler('json_rpc')
def rpc_hander(rq, method, jsonrpc, request_id, **values):
    jrpc = {
        'x': {
            'mimetype': 'application/json',
            'value': {
                'id': request_id['value'],
                'method': method['value'],
                'jsonrpc': jsonrpc['value'],
                'params': {name: val['value'] for name, val in values.items()}
            }
        }
    }
    render_variables(rq, 'data', **jrpc)

def random_id_gen():
    return uuid.uuid4().hex

def Kodi(hostname, username='kodi', password=None, port=8080):
    get_rpc = API(
        base_hive,
        hostname=hostname,
        port=port,
        username=username,
        password=password,
        request_id=random_id_gen
    )
    hive = Hive(get_rpc.API.get())
    return API(
        hive,
        hostname=hostname,
        port=port,
        username=username,
        password=password,
        request_id=random_id_gen
    )



import re
from urllib.parse import urljoin

from sickchill import logger
from sickchill.helper.common import try_int
from sickchill.oldbeard import tvcache
from sickchill.oldbeard.bs4_parser import BS4Parser
from sickchill.providers.torrent.TorrentProvider import TorrentProvider


class Provider(TorrentProvider):
    def __init__(self):

        super().__init__("HorribleSubs")

        self.public = True
        self.supports_absolute_numbering = True
        self.anime_only = True

        self.minseed = 0
        self.minleech = 0

        self.url = "https://horriblesubs.info"
        self.urls = {"search": urljoin(self.url, "api.php"), "rss": "http://www.horriblesubs.info/rss.php"}

        self.cache = tvcache.TVCache(self, min_time=15)  # only poll HorribleSubs every 15 minutes max

    def search(self, search_strings, age=0, ep_obj=None):
        results = []
        if self.show and not self.show.is_anime:
            return results

        for mode in search_strings:
            items = []
            logger.debug(_("Search Mode: {mode}".format(mode=mode)))

            for search_string in {*search_strings[mode]}:
                if mode == "RSS":
                    entries = self.__rssFeed()
                else:
                    entries = self.__getShow(search_string)

                items.extend(entries)

            # For each search mode sort all the items by seeders if available
            items.sort(key=lambda d: try_int(d.get("seeders", 0)), reverse=True)
            results.extend(items)

        return results

    def __rssFeed(self):
        entries = []

        rss_params = {"res": "all"}

        target_url = self.urls["rss"]

        data = self.get_url(target_url, params=rss_params, returns="text")
        if not data:
            return entries

        entries = self.__parseRssFeed(data)

        return entries

    @staticmethod
    def __parseRssFeed(data):
        entries = []
        with BS4Parser(data, "html5lib") as soup:
            items = soup.findAll("item")

            for item in items:
                title = item.find("title").text
                download_url = item.find("link").text

                entry = {"title": title, "link": download_url, "size": 333, "seeders": 1, "leechers": 1, "hash": ""}
                logger.debug(_("Found result: {title}".format(title=title)))

                entries.append(entry)

        return entries

    def __getShow(self, search_string):
        entries = []

        search_params = {"method": "search", "value": search_string}

        logger.debug(_("Search String: {search_string}".format(search_string=search_string)))
        target_url = self.urls["search"]

        data = self.get_url(target_url, params=search_params, returns="text")
        if not data:
            return entries

        entries = self.__parseSearchResult(data, target_url)

        return entries

    def __parseSearchResult(self, data, target_url):
        results = []
        with BS4Parser(data, "html5lib") as soup:
            lists = soup.find_all("ul")

            list_items = []
            for ul_list in lists:
                curr_list_item = ul_list("li") if ul_list else []
                list_items.extend(curr_list_item)

            # Continue only if one Release is found
            if len(list_items) < 1:
                logger.debug("Data returned from provider does not contain any torrents")
                return []

            for list_item in list_items:
                title = "{0}{1}".format(str(list_item.find("span").next_sibling), str(list_item.find("strong").text))
                logger.debug("Found title {0}".format(title))
                episode_url = "/#".join(list_item.find("a")["href"].rsplit("#", 1))
                episode = episode_url.split("#", 1)[1]

                page_url = "{0}{1}".format(self.url, episode_url)
                show_id = self.__getShowId(page_url)

                if not show_id:
                    logger.debug("Could not find show ID")
                    continue

                fetch_params = {"method": "getshows", "type": "show", "mode": "filter", "showid": show_id, "value": episode}

                entries = self.__fetchUrls(target_url, fetch_params, title)
                results.extend(entries)

        return results

    def __getShowId(self, target_url):
        data = self.get_url(target_url, returns="text")
        if not data:
            logger.debug("Could not fetch url: {0}".format(target_url))
            return None

        with BS4Parser(data, "html5lib") as soup:
            show_id = re.sub(r"[^0-9]", "", soup(text=re.compile("hs_showid"))[0])
            logger.debug("show id: {0}".format(show_id))

        return show_id

    def __fetchUrls(self, target_url, params, title):
        entries = []

        data = self.get_url(target_url, params=params, returns="text")
        if not data:
            return entries

        with BS4Parser(data, "html5lib") as soup:
            for div in soup.findAll("div", attrs={"class": "rls-link"}):
                quality = div.find("span", attrs={"class": "rls-link-label"}).get_text(strip=True)

                link = div.find("span", class_="hs-torrent-link")
                download_url = link.find("a")["href"] if link and link.find("a") else None

                if not download_url:
                    # fallback to magnet link
                    link = div.find("span", class_="hs-magnet-link")
                    download_url = link.find("a")["href"] if link and link.find("a") else None

                release_title = "[HorribleSubs] {0}.[{1}]".format(title, quality)
                item = {"title": release_title, "link": download_url, "size": 333, "seeders": 1, "leechers": 1, "hash": ""}
                logger.debug(_("Found result: ") + f"{release_title}")

                entries.append(item)

        return entries



# Copyright (c) 2013 Chris Lucas, <chris@chrisjlucas.com>
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import hashlib
import os.path
import re
from typing import Dict, Iterable, Union
from urllib.request import urlopen

import bencodepy


class TorrentParser(object):
    def __init__(self, torrent):
        """Decode and parse given torrent

        @param torrent: handles: urls, file paths, string of torrent data
        @type torrent: str

        @raise AssertionError: Can be raised for a couple reasons:
                               - If _get_raw_torrent() couldn't figure out
                               what X{torrent} is
                               - if X{torrent} isn't a valid bencoded torrent file
        """
        self.torrent = torrent
        self._raw_torrent = None  # : testing yo
        self._torrent_decoded = None  # : what up
        self.file_type = None

        self._get_raw_torrent()
        assert self._raw_torrent is not None, "Couldn't get raw_torrent."
        if self._torrent_decoded is None:
            self._decode_torrent()
        assert isinstance(self._torrent_decoded, dict), "Invalid torrent file."
        self._parse_torrent()

    def _is_raw(self):
        raw = False
        if isinstance(self.torrent, (str, bytes)):
            if isinstance(self._decode_torrent(self.torrent), dict):
                raw = True
            else:
                # reset self._torrent_decoded (currently equals False)
                self._torrent_decoded = None

        return raw

    def _get_raw_torrent(self):
        """Get raw torrent data by determining what self.torrent is"""
        # already raw?
        if self._is_raw():
            self.file_type = "raw"
            self._raw_torrent = self.torrent
            return
        # local file?
        if os.path.isfile(self.torrent):
            self.file_type = "file"
            self._raw_torrent = open(self.torrent, "rb").read()
        # url?
        elif re.search(r"^(http|ftp)://", self.torrent, re.I):
            self.file_type = "url"
            self._raw_torrent = urlopen(self.torrent).read()

    def _decode_torrent(self, raw_torrent=None):
        if raw_torrent is None:
            raw_torrent = self._raw_torrent
        self._torrent_decoded = bencodepy.decode(raw_torrent)
        return self._torrent_decoded

    def _calc_info_hash(self):
        self.info_hash = None
        if b"info" in self._torrent_decoded:
            info_encoded = bencodepy.encode(self._torrent_decoded[b"info"])

            if info_encoded:
                self.info_hash = hashlib.sha1(info_encoded).hexdigest().upper()

        return self.info_hash

    def _parse_torrent(self):
        for k in self._torrent_decoded:
            key = k.replace(b" ", b"_").lower()
            setattr(self, str(key), self._torrent_decoded[k])

        self._calc_info_hash()


from tornado.escape import xhtml_unescape
from tornado.web import addslash

from sickchill import settings
from sickchill.oldbeard import config
from sickchill.views.common import PageTemplate
from sickchill.views.home import Home
from sickchill.views.routes import Route


@Route("/home/postprocess(/?.*)", name="home:postprocess")
class PostProcess(Home):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @addslash
    def index(self, *args_, **kwargs_):
        t = PageTemplate(rh=self, filename="home_postprocess.mako")
        return t.render(title=_("Post Processing"), header=_("Post Processing"), topmenu="home", controller="home", action="postProcess")

    def processEpisode(
        self,
        proc_dir=None,
        nzbName=None,
        quiet=None,
        process_method=None,
        force=None,
        is_priority=None,
        delete_on="0",
        failed="0",
        proc_type="manual",
        force_next=False,
        *args_,
        **kwargs,
    ):

        mode = kwargs.get("type", proc_type)
        process_path = xhtml_unescape(kwargs.get("dir", proc_dir or "") or "")
        if not process_path:
            return self.redirect("/home/postprocess/")

        release_name = xhtml_unescape(nzbName) if nzbName else nzbName

        result = settings.postProcessorTaskScheduler.action.add_item(
            process_path,
            release_name,
            method=process_method,
            force=force,
            is_priority=is_priority,
            delete=delete_on,
            failed=failed,
            mode=mode,
            force_next=force_next,
        )

        if config.checkbox_to_value(quiet):
            return result

        if result:
            result = result.replace("\n", "<br>\n")

        return self._genericMessage("Postprocessing results", result)



import os

from tornado.web import addslash

import sickchill.start
from sickchill import settings
from sickchill.oldbeard import config, filters, ui
from sickchill.views.common import PageTemplate
from sickchill.views.routes import Route

from .index import Config


@Route("/config/anime(/?.*)", name="config:anime")
class ConfigAnime(Config):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @addslash
    def index(self, *args_, **kwargs_):

        t = PageTemplate(rh=self, filename="config_anime.mako")

        return t.render(submenu=self.ConfigMenu(), title=_("Config - Anime"), header=_("Anime"), topmenu="config", controller="config", action="anime")

    def saveAnime(self, use_anidb=None, anidb_username=None, anidb_password=None, anidb_use_mylist=None, split_home=None, split_home_in_tabs=None):

        settings.USE_ANIDB = config.checkbox_to_value(use_anidb)
        settings.ANIDB_USERNAME = anidb_username
        settings.ANIDB_PASSWORD = filters.unhide(settings.ANIDB_PASSWORD, anidb_password)
        settings.ANIDB_USE_MYLIST = config.checkbox_to_value(anidb_use_mylist)
        settings.ANIME_SPLIT_HOME = config.checkbox_to_value(split_home)
        settings.ANIME_SPLIT_HOME_IN_TABS = config.checkbox_to_value(split_home_in_tabs)

        sickchill.start.save_config()
        ui.notifications.message(_("Configuration Saved"), os.path.join(settings.CONFIG_FILE))

        return self.redirect("/config/anime/")



from typing import TYPE_CHECKING
from urllib.parse import splitport

import qbittorrentapi

from sickchill import settings
from sickchill.oldbeard.clients.generic import GenericClient

if TYPE_CHECKING:  # pragma: no cover
    from sickchill.oldbeard.classes import TorrentSearchResult


class Client(GenericClient):
    def __init__(self, host: str = None, username: str = None, password: str = None):
        super().__init__("qBittorrent", host, username, password)
        self.host, self.port = splitport(self.host or settings.TORRENT_HOST)
        self.api = qbittorrentapi.Client(
            host=self.host,
            port=self.port or None,
            username=self.username or settings.TORRENT_USERNAME,
            password=self.password or settings.TORRENT_PASSWORD,
            VERIFY_WEBUI_CERTIFICATE=settings.TORRENT_VERIFY_CERT,
        )

    def _get_auth(self):
        try:
            if not self.api.is_logged_in:
                self.api.auth_log_in()
        except (qbittorrentapi.LoginFailed, qbittorrentapi.APIConnectionError):
            return False
        return True

    def testAuthentication(self):
        try:
            if not self.api.is_logged_in:
                self.api.auth_log_in()
        except (qbittorrentapi.LoginFailed, qbittorrentapi.APIConnectionError) as error:
            return False, f"Failed to authenticate with {self.name}, {error}"
        return True, "Success: Connected and Authenticated"

    @staticmethod
    def __torrent_args(result: "TorrentSearchResult"):
        return dict(
            save_path=settings.TORRENT_DIR or None,
            category=(settings.TORRENT_LABEL, settings.TORRENT_LABEL_ANIME)[result.show.is_anime] or settings.TORRENT_LABEL,
            is_paused=settings.TORRENT_PAUSED,
        )

    def _add_torrent_uri(self, result: "TorrentSearchResult"):
        return self.api.torrents_add(urls=[result.url], **self.__torrent_args(result))

    def _add_torrent_file(self, result: "TorrentSearchResult"):
        return self.api.torrents_add(torrent_files=[result.content], **self.__torrent_args(result))

    def _set_torrent_priority(self, result: "TorrentSearchResult"):
        if not self.api.app_preferences().get("queueing_enabled"):
            return True
        return (self.api.torrents_decrease_priority, self.api.torrents_increase_priority)[result.priority == 1](result.hash.lower)



# -*- coding: utf-8 -*-
"""
Deprecated Library
==================

Python ``@deprecated`` decorator to deprecate old python classes, functions or methods.

"""

#: Module Version Number, see `PEP 396 <https://www.python.org/dev/peps/pep-0396/>`_.
__version__ = "1.2.10"

from deprecated.classic import deprecated



"""
This module provides methods that parse variables into
one of the three base level variable types. We map the
variable type names to methods that can handle them. Each
method consumes a Request object 'rq' as the first positional
argument, and does its work by calling one of the variable-setting
methods on rq.
"""

from __future__ import absolute_import, division
from __future__ import unicode_literals, print_function

from uuid import uuid4
from functools import partial
from base64 import b64encode

from beekeeper.data_handlers import encode
from beekeeper.exceptions import CannotHandleVariableTypes

class VariableHandler(object):

    registry = {}

    def __init__(self, *var_types):
        self.var_types = var_types

    def __call__(self, func):
        def wrapped(rq, **values):
            return func(rq, **values)
        for each in self.var_types:
            VariableHandler.registry[each] = wrapped
        return wrapped

def identity(**values):
    return {name: val['value'] for name, val in values.items()}

def set_content_type(rq, mimetype):
    rq.set_headers(**{'Content-Type': mimetype})

@VariableHandler('data')
def render_data(rq, **data):
    for val in data.values():
        set_content_type(rq, val['mimetype'])
        rq.set_data(encode(val['value'], val['mimetype']))

@VariableHandler('http_form')
def http_form(rq, **values):
    form = {
        'x': {
            'mimetype': 'application/x-www-form-urlencoded',
            'value': {
                name: val['value'] for name, val in values.items()
            }
        }
    }
    render(rq, 'data', **form)

@VariableHandler('http_basic_auth')
def basic_auth(rq, **values):
    username = values.get('username', {}).get('value', '')
    password = values.get('password', {}).get('value', '')
    authinfo = b64encode("{}:{}".format(username, password).encode('utf-8'))
    authinfo = 'Basic {}'.format(authinfo.decode('utf-8'))
    rq.set_headers(Authorization=authinfo)

@VariableHandler('bearer_token')
def bearer(rq, **values):
    if len(values) > 1:
        raise Exception('Only one bearer token allowed')
    else:
        for token in values.values():
            text = 'Bearer {}'.format(token['value'])
            rq.set_headers(Authorization=text)

@VariableHandler('cookie')
def cookies(rq, **values):
    cookie = '; '.join([value['value'] for value in values.values()])
    rq.set_headers(Cookie=cookie)

@VariableHandler('multipart')
def multipart(rq, **values):
    frame = '\n--{}\nContent-Disposition: form-data; name="{}"'
    boundary = uuid4().hex
    files = [name for name, data in values.items() if 'mimetype' in data]
    output = bytes()
    for name, value in values.items():
        if name in files:
            fname = value.get('filename', getattr(value['value'], 'name', uuid4().hex))
            this_frame = frame + '; filename="{}"\nContent-Type: {}\n\n'
            this_data = encode(value['value'], value['mimetype'])
            args = (boundary, name, fname, value['mimetype'])
        else:
            this_frame = frame + '\n\n'
            this_data = value['value'].encode('ascii')
            args = (boundary, name)
        output += this_frame.format(*args).encode('ascii') + this_data
    output += '\n--{}--'.format(boundary).encode('ascii')
    rq.set_data(output)
    content_type_header = 'multipart/form-data; boundary={}'.format(boundary)
    set_content_type(rq, content_type_header)

@VariableHandler('header')
def header(rq, **values):
    rq.set_headers(**identity(**values))

@VariableHandler('url_replacement')
def replacement(rq, **values):
    rq.set_url_replacements(**identity(**values))

@VariableHandler('url_param')
def url_param(rq, **values):
    rq.set_url_params(**identity(**values))

def render(rq, var_type, **values):
    if var_type in VariableHandler.registry:
        variables = {val.pop('name', name): val for name, val in values.items()}
        VariableHandler.registry[var_type](rq, **variables)
    else:
        raise CannotHandleVariableTypes(var_type)



# -*- coding: utf-8 -*-
"""
SubRip's subtitle parser
"""

from pysrt.srtexc import InvalidItem, InvalidIndex
from pysrt.srttime import SubRipTime
from pysrt.comparablemixin import ComparableMixin
from pysrt.compat import str, is_py2
import re


class SubRipItem(ComparableMixin):
    """
    SubRipItem(index, start, end, text, position)

    index -> int: index of item in file. 0 by default.
    start, end -> SubRipTime or coercible.
    text -> unicode: text content for item.
    position -> unicode: raw srt/vtt "display coordinates" string
    """
    ITEM_PATTERN = str('%s\n%s --> %s%s\n%s\n')
    TIMESTAMP_SEPARATOR = '-->'

    def __init__(self, index=0, start=None, end=None, text='', position=''):
        try:
            self.index = int(index)
        except (TypeError, ValueError):  # try to cast as int, but it's not mandatory
            self.index = index

        self.start = SubRipTime.coerce(start or 0)
        self.end = SubRipTime.coerce(end or 0)
        self.position = str(position)
        self.text = str(text)

    @property
    def duration(self):
        return self.end - self.start

    @property
    def text_without_tags(self):
        RE_TAG = re.compile(r'<[^>]*?>')
        return RE_TAG.sub('', self.text)

    @property
    def characters_per_second(self):
        characters_count = len(self.text_without_tags.replace('\n', ''))
        try:
            return characters_count / (self.duration.ordinal / 1000.0)
        except ZeroDivisionError:
            return 0.0

    def __str__(self):
        position = ' %s' % self.position if self.position.strip() else ''
        return self.ITEM_PATTERN % (self.index, self.start, self.end,
                                    position, self.text)
    if is_py2:
        __unicode__ = __str__

        def __str__(self):
            raise NotImplementedError('Use unicode() instead!')

    def _cmpkey(self):
        return (self.start, self.end)

    def shift(self, *args, **kwargs):
        """
        shift(hours, minutes, seconds, milliseconds, ratio)

        Add given values to start and end attributes.
        All arguments are optional and have a default value of 0.
        """
        self.start.shift(*args, **kwargs)
        self.end.shift(*args, **kwargs)

    @classmethod
    def from_string(cls, source):
        return cls.from_lines(source.splitlines(True))

    @classmethod
    def from_lines(cls, lines):
        if len(lines) < 2:
            raise InvalidItem()
        lines = [l.rstrip() for l in lines]
        index = None
        if cls.TIMESTAMP_SEPARATOR not in lines[0]:
            index = lines.pop(0)
        start, end, position = cls.split_timestamps(lines[0])
        body = '\n'.join(lines[1:])
        return cls(index, start, end, body, position)

    @classmethod
    def split_timestamps(cls, line):
        timestamps = line.split(cls.TIMESTAMP_SEPARATOR)
        if len(timestamps) != 2:
            raise InvalidItem()
        start, end_and_position = timestamps
        end_and_position = end_and_position.lstrip().split(' ', 1)
        end = end_and_position[0]
        position = end_and_position[1] if len(end_and_position) > 1 else ''
        return (s.strip() for s in (start, end, position))



# -*- coding: utf-8 -*-
import logging

from babelfish import Language, language_converters
from requests import Session

from . import Provider
from .. import __short_version__
from ..subtitle import Subtitle, fix_line_ending

logger = logging.getLogger(__name__)

language_converters.register('thesubdb = subliminal.converters.thesubdb:TheSubDBConverter')


class TheSubDBSubtitle(Subtitle):
    """TheSubDB Subtitle."""
    provider_name = 'thesubdb'

    def __init__(self, language, hash):
        super(TheSubDBSubtitle, self).__init__(language)
        self.hash = hash

    @property
    def id(self):
        return self.hash + '-' + str(self.language)

    @property
    def info(self):
        return self.hash

    def get_matches(self, video):
        matches = set()

        # hash
        if 'thesubdb' in video.hashes and video.hashes['thesubdb'] == self.hash:
            matches.add('hash')

        return matches


class TheSubDBProvider(Provider):
    """TheSubDB Provider."""
    languages = {Language.fromthesubdb(l) for l in language_converters['thesubdb'].codes}
    required_hash = 'thesubdb'
    server_url = 'http://api.thesubdb.com/'
    subtitle_class = TheSubDBSubtitle
    user_agent = 'SubDB/1.0 (subliminal/%s; https://github.com/Diaoul/subliminal)' % __short_version__

    def __init__(self):
        self.session = None

    def initialize(self):
        self.session = Session()
        self.session.headers['User-Agent'] = self.user_agent

    def terminate(self):
        self.session.close()

    def query(self, hash):
        # make the query
        params = {'action': 'search', 'hash': hash}
        logger.info('Searching subtitles %r', params)
        r = self.session.get(self.server_url, params=params, timeout=10)

        # handle subtitles not found and errors
        if r.status_code == 404:
            logger.debug('No subtitles found')
            return []
        r.raise_for_status()

        # loop over languages
        subtitles = []
        for language_code in r.text.split(','):
            language = Language.fromthesubdb(language_code)

            subtitle = self.subtitle_class(language, hash)
            logger.debug('Found subtitle %r', subtitle)
            subtitles.append(subtitle)

        return subtitles

    def list_subtitles(self, video, languages):
        return [s for s in self.query(video.hashes['thesubdb']) if s.language in languages]

    def download_subtitle(self, subtitle):
        logger.info('Downloading subtitle %r', subtitle)
        params = {'action': 'download', 'hash': subtitle.hash, 'language': subtitle.language.alpha2}
        r = self.session.get(self.server_url, params=params, timeout=10)
        r.raise_for_status()

        subtitle.content = fix_line_ending(r.content)



import re

from .utils import validator

pattern = re.compile(r"^[13][a-km-zA-HJ-NP-Z1-9]{25,34}$")


@validator
def btc_address(value):
    """
    Return whether or not given value is a valid bitcoin address.

    If the value is valid bitcoin address this function returns ``True``,
    otherwise :class:`~validators.utils.ValidationFailure`.

    Examples::

        >>> btc_address('3Cwgr2g7vsi1bXDUkpEnVoRLA9w4FZfC69')
        True

    :param value: Bitcoin address string to validate
    """
    return pattern.match(value)



# -*- coding: utf-8 -*-
import logging

from babelfish import Language
from requests import Session

from . import Provider
from ..subtitle import Subtitle

logger = logging.getLogger(__name__)


def get_subhash(hash):
    """Get a second hash based on napiprojekt's hash.

    :param str hash: napiprojekt's hash.
    :return: the subhash.
    :rtype: str

    """
    idx = [0xe, 0x3, 0x6, 0x8, 0x2]
    mul = [2, 2, 5, 4, 3]
    add = [0, 0xd, 0x10, 0xb, 0x5]

    b = []
    for i in range(len(idx)):
        a = add[i]
        m = mul[i]
        i = idx[i]
        t = a + int(hash[i], 16)
        v = int(hash[t:t + 2], 16)
        b.append(('%x' % (v * m))[-1])

    return ''.join(b)


class NapiProjektSubtitle(Subtitle):
    """NapiProjekt Subtitle."""
    provider_name = 'napiprojekt'

    def __init__(self, language, hash):
        super(NapiProjektSubtitle, self).__init__(language)
        self.hash = hash
        self.content = None

    @property
    def id(self):
        return self.hash

    @property
    def info(self):
        return self.hash

    def get_matches(self, video):
        matches = set()

        # hash
        if 'napiprojekt' in video.hashes and video.hashes['napiprojekt'] == self.hash:
            matches.add('hash')

        return matches


class NapiProjektProvider(Provider):
    """NapiProjekt Provider."""
    languages = {Language.fromalpha2(l) for l in ['pl']}
    required_hash = 'napiprojekt'
    server_url = 'http://napiprojekt.pl/unit_napisy/dl.php'
    subtitle_class = NapiProjektSubtitle

    def __init__(self):
        self.session = None

    def initialize(self):
        self.session = Session()
        self.session.headers['User-Agent'] = self.user_agent

    def terminate(self):
        self.session.close()

    def query(self, language, hash):
        params = {
            'v': 'dreambox',
            'kolejka': 'false',
            'nick': '',
            'pass': '',
            'napios': 'Linux',
            'l': language.alpha2.upper(),
            'f': hash,
            't': get_subhash(hash)}
        logger.info('Searching subtitle %r', params)
        r = self.session.get(self.server_url, params=params, timeout=10)
        r.raise_for_status()

        # handle subtitles not found and errors
        if r.content[:4] == b'NPc0':
            logger.debug('No subtitles found')
            return None

        subtitle = self.subtitle_class(language, hash)
        subtitle.content = r.content
        logger.debug('Found subtitle %r', subtitle)

        return subtitle

    def list_subtitles(self, video, languages):
        return [s for s in [self.query(l, video.hashes['napiprojekt']) for l in languages] if s is not None]

    def download_subtitle(self, subtitle):
        # there is no download step, content is already filled from listing subtitles
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2016-5-26

@author: hustcc
'''
from __future__ import absolute_import

from timeago import setting


def locale_module(mod, locale):
    try:
        return getattr(getattr(getattr(mod, 'locales'), locale), 'LOCALE')
    except:
        raise


def timeago_template(locale, index, ago_in):
    '''
    simple locale implement
    '''
    try:
        LOCALE = __import__('timeago.locales.' + locale)
        LOCALE = locale_module(LOCALE, locale)
    except:
        locale = setting.DEFAULT_LOCALE
        LOCALE = __import__('timeago.locales.' + locale)
        LOCALE = locale_module(LOCALE, locale)

    if isinstance(LOCALE, list):
        return LOCALE[index][ago_in]
    else:
        return LOCALE(index, ago_in)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: avrong
'''

base = [
    ["только что", "через несколько секунд"],
    ["%s секунду назад", "через %s секунду", "%s секунды назад", "через %s секунды", "%s секунд назад", "через %s секунд"],
    ["минуту назад", "через минуту"],
    ["%s минуту назад", "через %s минуту", "%s минуты назад", "через %s минуты", "%s минут назад", "через %s минут"],
    ["час назад", "через час"],
    ["%s час назад", "через %s час", "%s часа назад", "через %s часа", "%s часов назад", "через %s часов"],
    ["вчера", "завтра"],
    ["%s день назад", "через %s день", "%s дня назад", "через %s дня", "%s дней назад", "через %s дней"],
    ["неделю назад", "через неделю"],
    ["%s неделю назад", "через %s неделю", "%s недели назад", "через %s недели", "%s недель назад", "через %s недель"],
    ["месяц назад", "через месяц"],
    ["%s месяц назад", "через %s месяц", "%s месяца назад", "через %s месяца", "%s месяцев назад", "через %s месяцев"],
    ["год назад", "через год"],
    ["%s год назад", "через %s год", "%s года назад", "через %s года", "%s лет назад", "через %s лет"],
]


def generate(row, y):
    def formatting(time):
        '''Uses the first and second fields for (2,3,4) cases and 3rd and 4th for the rest.'''
        multiple_rows = [
            1,   # multiple seconds
            3,   # multiple minutes
            5,   # multiple hours
            7,   # multiple days
            9,   # multiple weeks
            11,  # multiple months
            13,  # multiple years
        ]
        if row in multiple_rows:
            if time > 20:
                time = time % 10
            if time == 1:
                return base[row][y]
            elif time in (2, 3, 4):
                return base[row][y + 2]
            else:
                return base[row][y + 4]
        return base[row][y]

    return formatting


LOCALE = generate



from datetime import date

from sickchill import settings
from sickchill.helper.exceptions import CantRefreshShowException, CantRemoveShowException, MultipleShowObjectsException
from sickchill.oldbeard.common import Quality, SKIPPED, WANTED
from sickchill.oldbeard.db import DBConnection


class Show(object):
    def __init__(self):
        pass

    @staticmethod
    def delete(indexer_id, remove_files=False):
        """
        Try to delete a show
        :param indexer_id: The unique id of the show to delete
        :param remove_files: ``True`` to remove the files associated with the show, ``False`` otherwise
        :return: A tuple containing:
         - an error message if the show could not be deleted, ``None`` otherwise
         - the show object that was deleted, if it exists, ``None`` otherwise
        """

        error, show = Show._validate_indexer_id(indexer_id)

        if error is not None:
            return error, show

        if show:
            try:
                settings.showQueueScheduler.action.remove_show(show, bool(remove_files))
            except CantRemoveShowException as exception:
                return str(exception), show

        return None, show

    @staticmethod
    def find(shows, indexer_id):
        """
        Find a show by its indexer id in the provided list of shows
        :param shows: The list of shows to search in
        :param indexer_id: The indexer id of the desired show
        :return: The desired show if found, ``None`` if not found
        :throw: ``MultipleShowObjectsException`` if multiple shows match the provided ``indexer_id``
        """
        if not indexer_id or not shows:
            return None

        if not isinstance(indexer_id, (str, int)):
            return None

        if isinstance(indexer_id, list):
            if not isinstance(indexer_id[0], (int, str)):
                return None

            indexer_ids = [int(x) for x in indexer_id]
        else:
            indexer_ids = [int(indexer_id)]

        results = [show for show in shows if show.indexerid in indexer_ids]

        if not results:
            return None

        if len(results) == 1:
            return results[0]

        raise MultipleShowObjectsException()

    @staticmethod
    def overall_stats():
        db = DBConnection()
        shows = settings.showList
        today = date.today().toordinal()

        downloaded_status = Quality.DOWNLOADED + Quality.ARCHIVED
        snatched_status = Quality.SNATCHED + Quality.SNATCHED_PROPER + Quality.SNATCHED_BEST
        total_status = [SKIPPED, WANTED]

        results = db.select("SELECT airdate, status " "FROM tv_episodes " "WHERE season > 0 " "AND episode > 0 " "AND airdate > 1")

        stats = {
            "episodes": {
                "downloaded": 0,
                "snatched": 0,
                "total": 0,
            },
            "shows": {
                "active": len([show for show in shows if show.paused == 0 and show.status == "Continuing"]),
                "total": len(shows),
            },
        }

        for result in results:
            if result["status"] in downloaded_status:
                stats["episodes"]["downloaded"] += 1
                stats["episodes"]["total"] += 1
            elif result["status"] in snatched_status:
                stats["episodes"]["snatched"] += 1
                stats["episodes"]["total"] += 1
            elif result["airdate"] <= today and result["status"] in total_status:
                stats["episodes"]["total"] += 1

        return stats

    @staticmethod
    def pause(indexer_id, pause=None):
        """
        Change the pause state of a show
        :param indexer_id: The unique id of the show to update
        :param pause: ``True`` to pause the show, ``False`` to resume the show, ``None`` to toggle the pause state
        :return: A tuple containing:
         - an error message if the pause state could not be changed, ``None`` otherwise
         - the show object that was updated, if it exists, ``None`` otherwise
        """

        error, show = Show._validate_indexer_id(indexer_id)

        if error is not None:
            return error, show

        if pause is None:
            show.paused = not show.paused
        else:
            show.paused = pause

        show.saveToDB()

        return None, show

    @staticmethod
    def refresh(indexer_id, force=False):
        """
        Try to refresh a show
        :param force: Force refresh
        :param indexer_id: The unique id of the show to refresh
        :return: A tuple containing:
         - an error message if the show could not be refreshed, ``None`` otherwise
         - the show object that was refreshed, if it exists, ``None`` otherwise
        """

        error, show = Show._validate_indexer_id(indexer_id)

        if error is not None:
            return error, show

        try:
            settings.showQueueScheduler.action.refresh_show(show, force)
        except CantRefreshShowException as exception:
            return str(exception), show

        return None, show

    @staticmethod
    def _validate_indexer_id(indexer_id):
        """
        Check that the provided indexer_id is valid and corresponds with a known show
        :param indexer_id: The indexer id to check
        :return: A tuple containing:
         - an error message if the indexer id is not correct, ``None`` otherwise
         - the show object corresponding to ``indexer_id`` if it exists, ``None`` otherwise
        """

        try:
            indexer_id = int(indexer_id)
        except (TypeError, ValueError):
            return "Invalid show ID", None

        try:
            show = Show.find(settings.showList, indexer_id)
        except MultipleShowObjectsException:
            return "Unable to find the specified show", None

        return None, show



import re
from xml.etree import ElementTree

import sickchill.oldbeard.common
from sickchill import logger, settings
from sickchill.oldbeard import common
from sickchill.oldbeard.helpers import getURL, make_session


class Notifier(object):
    def __init__(self):
        self.headers = {
            "X-Plex-Device-Name": "SickChill",
            "X-Plex-Product": "SickChill Notifier",
            "X-Plex-Client-Identifier": sickchill.oldbeard.common.USER_AGENT,
            "X-Plex-Version": "2016.02.10",
        }
        self.session = make_session()

    @staticmethod
    def _notify_pht(message, title="SickChill", host=None, username=None, password=None, force=False):
        """Internal wrapper for the notify_snatch and notify_download functions

        Args:
            message: Message body of the notice to send
            title: Title of the notice to send
            host: Plex Home Theater(s) host:port
            username: Plex username
            password: Plex password
            force: Used for the Test method to override config safety checks

        Returns:
            Returns a list results in the format of host:ip:result
            The result will either be 'OK' or False, this is used to be parsed by the calling function.

        """

        # suppress notifications if the notifier is disabled but the notify options are checked
        if not settings.USE_PLEX_CLIENT and not force:
            return False

        host = host or settings.PLEX_CLIENT_HOST
        username = username or settings.PLEX_CLIENT_USERNAME
        password = password or settings.PLEX_CLIENT_PASSWORD

        return sickchill.oldbeard.notifiers.kodi_notifier._notify_kodi(
            message, title=title, host=host, username=username, password=password, force=force, dest_app="PLEX"
        )

    ##############################################################################
    # Public functions
    ##############################################################################

    def notify_snatch(self, ep_name):
        if settings.PLEX_NOTIFY_ONSNATCH:
            self._notify_pht(ep_name, common.notifyStrings[common.NOTIFY_SNATCH])

    def notify_download(self, ep_name):
        if settings.PLEX_NOTIFY_ONDOWNLOAD:
            self._notify_pht(ep_name, common.notifyStrings[common.NOTIFY_DOWNLOAD])

    def notify_subtitle_download(self, ep_name, lang):
        if settings.PLEX_NOTIFY_ONSUBTITLEDOWNLOAD:
            self._notify_pht(ep_name + ": " + lang, common.notifyStrings[common.NOTIFY_SUBTITLE_DOWNLOAD])

    def notify_git_update(self, new_version="??"):
        if settings.NOTIFY_ON_UPDATE:
            update_text = common.notifyStrings[common.NOTIFY_GIT_UPDATE_TEXT]
            title = common.notifyStrings[common.NOTIFY_GIT_UPDATE]
            if update_text and title and new_version:
                self._notify_pht(update_text + new_version, title)

    def notify_login(self, ipaddress=""):
        if settings.NOTIFY_ON_LOGIN:
            update_text = common.notifyStrings[common.NOTIFY_LOGIN_TEXT]
            title = common.notifyStrings[common.NOTIFY_LOGIN]
            if update_text and title and ipaddress:
                self._notify_pht(update_text.format(ipaddress), title)

    def test_notify_pht(self, host, username, password):
        return self._notify_pht("This is a test notification from SickChill", "Test Notification", host, username, password, force=True)

    def test_notify_pms(self, host, username, password, plex_server_token):
        return self.update_library(host=host, username=username, password=password, plex_server_token=plex_server_token, force=True)

    def update_library(self, ep_obj=None, host=None, username=None, password=None, plex_server_token=None, force=False):
        """Handles updating the Plex Media Server host via HTTP API

        Plex Media Server currently only supports updating the whole video library and not a specific path.

        Returns:
            Returns None for no issue, else a string of host with connection issues

        """

        if not (settings.USE_PLEX_SERVER and settings.PLEX_UPDATE_LIBRARY) and not force:
            return None

        host = host or settings.PLEX_SERVER_HOST
        if not host:
            logger.debug("PLEX: No Plex Media Server host specified, check your settings")
            return False

        if not self.get_token(username, password, plex_server_token):
            logger.warning("PLEX: Error getting auth token for Plex Media Server, check your settings")
            return False

        file_location = "" if not ep_obj else ep_obj.location
        host_list = {x.strip() for x in host.split(",") if x.strip()}
        hosts_all = hosts_match = {}
        hosts_failed = set()

        for cur_host in host_list:

            url = "http{0}://{1}/library/sections".format(("", "s")[settings.PLEX_SERVER_HTTPS], cur_host)
            try:
                xml_response = getURL(url, headers=self.headers, session=self.session, returns="text", verify=False, allow_proxy=False)
                if not xml_response:
                    logger.warning("PLEX: Error while trying to contact Plex Media Server: {0}".format(cur_host))
                    hosts_failed.add(cur_host)
                    continue

                media_container = ElementTree.fromstring(xml_response)
            except IOError as error:
                logger.warning("PLEX: Error while trying to contact Plex Media Server: {0}".format(str(error)))
                hosts_failed.add(cur_host)
                continue
            except Exception as error:
                if "invalid token" in str(error):
                    logger.warning("PLEX: Please set TOKEN in Plex settings: ")
                else:
                    logger.warning("PLEX: Error while trying to contact Plex Media Server: {0}".format(str(error)))
                hosts_failed.add(cur_host)
                continue

            sections = media_container.findall(".//Directory")
            if not sections:
                logger.debug("PLEX: Plex Media Server not running on: {0}".format(cur_host))
                hosts_failed.add(cur_host)
                continue

            for section in sections:
                if "show" == section.attrib["type"]:

                    keyed_host = [(str(section.attrib["key"]), cur_host)]
                    hosts_all.update(keyed_host)
                    if not file_location:
                        continue

                    for section_location in section.findall(".//Location"):
                        section_path = re.sub(r"[/\\]+", "/", section_location.attrib["path"].lower())
                        section_path = re.sub(r"^(.{,2})[/\\]", "", section_path)
                        location_path = re.sub(r"[/\\]+", "/", file_location.lower())
                        location_path = re.sub(r"^(.{,2})[/\\]", "", location_path)

                        if section_path in location_path:
                            hosts_match.update(keyed_host)

        if force:
            return (", ".join(set(hosts_failed)), None)[not len(hosts_failed)]

        if hosts_match:
            logger.debug("PLEX: Updating hosts where TV section paths match the downloaded show: " + ", ".join(set(hosts_match)))
        else:
            logger.debug("PLEX: Updating all hosts with TV sections: " + ", ".join(set(hosts_all)))

        hosts_try = (hosts_match.copy(), hosts_all.copy())[not len(hosts_match)]
        for section_key, cur_host in hosts_try.items():

            url = "http{0}://{1}/library/sections/{2}/refresh".format(("", "s")[settings.PLEX_SERVER_HTTPS], cur_host, section_key)
            try:
                getURL(url, headers=self.headers, session=self.session, returns="text", verify=False, allow_proxy=False)
            except Exception as error:
                logger.warning("PLEX: Error updating library section for Plex Media Server: {0}".format(str(error)))
                hosts_failed.add(cur_host)

        return (", ".join(set(hosts_failed)), None)[not len(hosts_failed)]

    def get_token(self, username=None, password=None, plex_server_token=None):
        username = username or settings.PLEX_SERVER_USERNAME
        password = password or settings.PLEX_SERVER_PASSWORD
        plex_server_token = plex_server_token or settings.PLEX_SERVER_TOKEN

        if plex_server_token:
            self.headers["X-Plex-Token"] = plex_server_token

        if "X-Plex-Token" in self.headers:
            return True

        if not (username and password):
            return True

        logger.debug("PLEX: fetching plex.tv credentials for user: " + username)

        params = {"user[login]": username, "user[password]": password}

        try:
            response = getURL(
                "https://plex.tv/users/sign_in.json", post_data=params, headers=self.headers, session=self.session, returns="json", allow_proxy=False
            )

            self.headers["X-Plex-Token"] = response["user"]["authentication_token"]

        except Exception as error:
            self.headers.pop("X-Plex-Token", "")
            logger.debug("PLEX: Error fetching credentials from from plex.tv for user {0}: {1}".format(username, error))

        return "X-Plex-Token" in self.headers



"""
Test XEM
"""

import unittest

import sickchill.oldbeard.db
from sickchill import settings
from sickchill.tv import TVShow
from tests import conftest


class XEMBasicTests(conftest.SickChillTestDBCase):
    """
    Perform basic xem tests
    """

    @staticmethod
    def load_shows_from_db():
        """
        Populates the showList with shows from the database
        """

        test_main_db_con = sickchill.oldbeard.db.DBConnection()
        sql_results = test_main_db_con.select("SELECT * FROM tv_shows")

        for sql_show in sql_results:
            try:
                cur_show = TVShow(int(sql_show["indexer"]), int(sql_show["indexer_id"]))
                settings.showList.append(cur_show)
            except Exception:  # noqa: S110
                pass

    @staticmethod
    def load_from_db():
        """
        Populates the showList with shows from the database
        """
        test_main_db_con = sickchill.oldbeard.db.DBConnection()
        sql_results = test_main_db_con.select("SELECT * FROM tv_shows")

        for sql_show in sql_results:
            try:
                cur_show = TVShow(int(sql_show["indexer"]), int(sql_show["indexer_id"]))
                settings.showList.append(cur_show)
            except Exception as error:
                print("There was an error creating the show {0}".format(error))

    # @staticmethod
    # def test_formatting():
    #     """
    #     Test formatting
    #     """
    #     name = "Game.of.Thrones.S03.720p.HDTV.x264-CtrlHD"
    #     release = "Game of Thrones"
    #
    #     # m = re.match('(?P<ep_ab_num>(?>\d{1,3})(?![ip])).+', name)
    #
    #     escaped_name = re.sub('\\\\[\\s.-]', '\W+', re.escape(release))
    #     cur_regex = '^' + escaped_name + r'\W+(?:(?:S\d[\dE._ -])|' + \
    #         r'(?:\d\d?x)|(?:\d{4}\W\d\d\W\d\d)|(?:(?:part|pt)[\._ -]?(\d|[ivx]))|' + \
    #         r'Season\W+\d+\W+|E\d+\W+|(?:\d{1,3}.+\d{1,}[a-zA-Z]{2}\W+[a-zA-Z]{3,}\W+\d{4}.+))'
    #
    #     # print("Checking if show " + name + " matches " + curRegex)
    #
    #     match = re.search(cur_regex, name, re.I)
    #     if match:
    #         # print("Matched " + curRegex + " to " + name)
    #         pass


if __name__ == "__main__":
    print("==================")
    print("STARTING - XEM Scene Numbering TESTS")
    print("==================")
    print("######################################################################")

    SUITE = unittest.TestLoader().loadTestsFromTestCase(XEMBasicTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
edition property
"""
from rebulk.remodule import re

from rebulk import Rebulk
from ..common import dash
from ..common.pattern import is_disabled
from ..common.validators import seps_surround


def edition(config):  # pylint:disable=unused-argument
    """
    Builder for rebulk object.

    :param config: rule configuration
    :type config: dict
    :return: Created Rebulk object
    :rtype: Rebulk
    """
    rebulk = Rebulk(disabled=lambda context: is_disabled(context, 'edition'))
    rebulk = rebulk.regex_defaults(flags=re.IGNORECASE, abbreviations=[dash]).string_defaults(ignore_case=True)
    rebulk.defaults(name='edition', validator=seps_surround)

    rebulk.regex('collector', "collector'?s?-edition", 'edition-collector', value='Collector')
    rebulk.regex('special-edition', 'edition-special', value='Special',
                 conflict_solver=lambda match, other: other
                 if other.name == 'episode_details' and other.value == 'Special'
                 else '__default__')
    rebulk.string('se', value='Special', tags='has-neighbor')
    rebulk.string('ddc', value="Director's Definitive Cut")
    rebulk.regex('criterion-edition', 'edition-criterion', 'CC', value='Criterion')
    rebulk.regex('deluxe', 'deluxe-edition', 'edition-deluxe', value='Deluxe')
    rebulk.regex('limited', 'limited-edition', value='Limited', tags=['has-neighbor', 'release-group-prefix'])
    rebulk.regex(r'theatrical-cut', r'theatrical-edition', r'theatrical', value='Theatrical')
    rebulk.regex(r"director'?s?-cut", r"director'?s?-cut-edition", r"edition-director'?s?-cut", 'DC',
                 value="Director's Cut")
    rebulk.regex('extended', 'extended-?cut', 'extended-?version',
                 value='Extended', tags=['has-neighbor', 'release-group-prefix'])
    rebulk.regex('alternat(e|ive)(?:-?Cut)?', value='Alternative Cut', tags=['has-neighbor', 'release-group-prefix'])
    for value in ('Remastered', 'Uncensored', 'Uncut', 'Unrated'):
        rebulk.string(value, value=value, tags=['has-neighbor', 'release-group-prefix'])
    rebulk.string('Festival', value='Festival', tags=['has-neighbor-before', 'has-neighbor-after'])
    rebulk.regex('imax', 'imax-edition', value='IMAX')
    rebulk.regex('fan-edit(?:ion)?', 'fan-collection', value='Fan')
    rebulk.regex('ultimate-edition', value='Ultimate')
    rebulk.regex("ultimate-collector'?s?-edition", value=['Ultimate', 'Collector'])
    rebulk.regex('ultimate-fan-edit(?:ion)?', 'ultimate-fan-collection', value=['Ultimate', 'Fan'])

    return rebulk



import json
from base64 import b64encode
from typing import TYPE_CHECKING

from sickchill import settings
from sickchill.oldbeard.clients.generic import GenericClient

if TYPE_CHECKING:  # pragma: no cover
    from sickchill.oldbeard.classes import TorrentSearchResult


class Client(GenericClient):
    def __init__(self, host: str = None, username: str = None, password: str = None):

        super().__init__("Transmission", host, username, password)
        self.url = "/".join((self.host.rstrip("/"), settings.TORRENT_RPCURL.strip("/"), "rpc"))

    def _get_auth(self):

        post_data = json.dumps(
            {
                "method": "session-get",
            }
        )

        try:
            self.response = self.session.post(self.url, data=post_data, timeout=120, verify=settings.TORRENT_VERIFY_CERT)
            self.auth = self.response.headers["X-Transmission-Session-Id"]
        except Exception:
            return None

        self.session.headers.update({"x-transmission-session-id": self.auth})

        # Validating Transmission authorization
        post_data = json.dumps({"arguments": {}, "method": "session-get"})

        self._request(method="post", data=post_data)

        return self.auth

    def _add_torrent_uri(self, result: "TorrentSearchResult"):
        self._request(method="post", data=self.__make_post(result, method="uri"))
        return self.response.json()["result"] == "success"

    def _add_torrent_file(self, result: "TorrentSearchResult"):
        self._request(method="post", data=self.__make_post(result, method="file"))
        return self.response.json()["result"] == "success"

    @staticmethod
    def __make_post(result, method="file"):
        arguments = {"paused": int(settings.TORRENT_PAUSED)}
        if method == "file":
            arguments.update({"metainfo": b64encode(result.content).decode("ascii")})
        else:
            arguments.update({"filename": result.url})

        if settings.TORRENT_PATH:
            arguments["download-dir"] = settings.TORRENT_PATH

        if settings.TORRENT_PATH_INCOMPLETE:
            arguments["incomplete-dir"] = settings.TORRENT_PATH_INCOMPLETE
            arguments["incomplete-enabled"] = 1

        ratio = None
        if result.ratio:
            ratio = result.ratio

        mode = 0
        if ratio:
            if float(ratio) == -1:
                ratio = 0
                mode = 2
            elif float(ratio) >= 0:
                ratio = float(ratio)
                mode = 1  # Stop seeding at seedRatioLimit

        arguments.update({"seedRatioLimit": ratio, "seedRatioMode": mode})

        if settings.TORRENT_SEED_TIME and settings.TORRENT_SEED_TIME != -1:
            time = int(60 * float(settings.TORRENT_SEED_TIME))
            arguments.update({"seedIdleLimit": time, "seedIdleMode": 1})

        if result.priority == -1:
            arguments["priority-low"] = []
        elif result.priority == 1:
            # set high priority for all files in torrent
            arguments["priority-high"] = []
            # move torrent to the top if the queue
            arguments["queuePosition"] = 0
            if settings.TORRENT_HIGH_BANDWIDTH:
                arguments["bandwidthPriority"] = 1
        else:
            arguments["priority-normal"] = []

        return json.dumps({"arguments": arguments, "method": "torrent-add"})



import datetime
import re
# noinspection PyUnresolvedReferences
import urllib.parse

from sickchill.helper.exceptions import EpisodeNotFoundException
from sickchill.show.History import History

from .. import logger
from . import db
from .common import FAILED, Quality, WANTED


def prepareFailedName(release):
    """Standardizes release name for failed DB"""

    fixed = urllib.parse.unquote(release)
    if fixed.endswith((".nzb", ".torrent")):
        fixed = fixed.rpartition(".")[0]

    fixed = re.sub(r"[\.\-\+\ ]", "_", fixed)

    return fixed


def logFailed(release):
    log_str = ""
    size = -1
    provider = ""

    release = prepareFailedName(release)

    failed_db_con = db.DBConnection("failed.db")
    sql_results = failed_db_con.select("SELECT * FROM history WHERE release = ?", [release])

    if not sql_results:
        logger.warning("Release not found in snatch history.")
    elif len(sql_results) > 1:
        logger.warning("Multiple logged snatches found for release")
        sizes = len(set(x["size"] for x in sql_results))
        providers = len(set(x["provider"] for x in sql_results))
        if sizes == 1:
            logger.warning("However, they're all the same size. Continuing with found size.")
            size = sql_results[0]["size"]
        else:
            logger.warning("They also vary in size. Deleting the logged snatches and recording this release with no size/provider")
            for result in sql_results:
                deleteLoggedSnatch(result["release"], result["size"], result["provider"])

        if providers == 1:
            logger.info("They're also from the same provider. Using it as well.")
            provider = sql_results[0]["provider"]
    else:
        size = sql_results[0]["size"]
        provider = sql_results[0]["provider"]

    if not hasFailed(release, size, provider):
        failed_db_con = db.DBConnection("failed.db")
        failed_db_con.action("INSERT INTO failed ('release', size, provider) VALUES (?, ?, ?)", [release, size, provider])

    deleteLoggedSnatch(release, size, provider)

    return log_str


def logSuccess(release):
    release = prepareFailedName(release)

    failed_db_con = db.DBConnection("failed.db")
    failed_db_con.action("DELETE FROM history WHERE release = ?", [release])


def hasFailed(release, size, provider="%"):
    """
    Returns True if a release has previously failed.

    If provider is given, return True only if the release is found
    with that specific provider. Otherwise, return True if the release
    is found with any provider.

    :param release: Release name to record failure
    :param size: Size of release
    :param provider: Specific provider to search (defaults to all providers)
    :return: True if a release has previously failed.
    """

    release = prepareFailedName(release)

    failed_db_con = db.DBConnection("failed.db")
    sql_results = failed_db_con.select("SELECT release FROM failed WHERE release=? AND size=? AND provider LIKE ? LIMIT 1", [release, size, provider])

    return len(sql_results) > 0


def revertEpisode(epObj):
    """Restore the episodes of a failed download to their original state"""
    failed_db_con = db.DBConnection("failed.db")
    sql_results = failed_db_con.select("SELECT episode, old_status FROM history WHERE showid=? AND season=?", [epObj.show.indexerid, epObj.season])

    history_eps = {res["episode"]: res for res in sql_results}

    try:
        logger.info("Reverting episode ({0}, {1}): {2}".format(epObj.season, epObj.episode, epObj.name))
        with epObj.lock:
            if epObj.episode in history_eps:
                logger.info("Found in history")
                epObj.status = history_eps[epObj.episode]["old_status"]
            else:
                logger.debug("Episode don't have a previous snatched status to revert. Setting it back to WANTED")
                epObj.status = WANTED
                epObj.saveToDB()

    except EpisodeNotFoundException as e:
        logger.warning("Unable to create episode, please set its status manually: " + str(e))


def markFailed(epObj):
    """
    Mark an episode as failed

    :param epObj: Episode object to mark as failed
    :return: empty string
    """
    log_str = ""

    try:
        with epObj.lock:
            quality = Quality.splitCompositeStatus(epObj.status)[1]
            epObj.status = Quality.compositeStatus(FAILED, quality)
            epObj.saveToDB()

    except EpisodeNotFoundException as e:
        logger.warning("Unable to get episode, please set its status manually: " + str(e))

    return log_str


def logSnatch(searchResult):
    """
    Logs a successful snatch

    :param searchResult: Search result that was successful
    """
    logDate = datetime.datetime.today().strftime(History.date_format)
    release = prepareFailedName(searchResult.name)

    providerClass = searchResult.provider
    if providerClass is not None:
        provider = providerClass.name
    else:
        provider = "unknown"

    show_obj = searchResult.episodes[0].show

    failed_db_con = db.DBConnection("failed.db")
    for episode in searchResult.episodes:
        failed_db_con.action(
            "INSERT INTO history (date, size, 'release', provider, showid, season, episode, old_status)" "VALUES (?, ?, ?, ?, ?, ?, ?, ?)",
            [logDate, searchResult.size, release, provider, show_obj.indexerid, episode.season, episode.episode, episode.status],
        )


def deleteLoggedSnatch(release, size, provider):
    """
    Remove a snatch from history

    :param release: release to delete
    :param size: Size of release
    :param provider: Provider to delete it from
    """
    release = prepareFailedName(release)

    failed_db_con = db.DBConnection("failed.db")
    failed_db_con.action("DELETE FROM history WHERE release=? AND size=? AND provider=?", [release, size, provider])


def trimHistory():
    """Trims history table to 1 month of history from today"""
    failed_db_con = db.DBConnection("failed.db")
    failed_db_con.action("DELETE FROM history WHERE date < " + str((datetime.datetime.today() - datetime.timedelta(days=30)).strftime(History.date_format)))


def findRelease(epObj):
    """
    Find releases in history by show ID and season.
    Return None for release if multiple found or no release found.
    """

    release = None
    provider = None

    # Clear old snatches for this release if any exist
    failed_db_con = db.DBConnection("failed.db")
    # failed_db_con.action(
    #     "DELETE FROM history WHERE showid = {0} AND season = {1} AND episode = {2}"
    #     " AND date < (SELECT max(date) FROM history WHERE showid = {0} AND season = {1} AND episode = {2})".format
    #     (epObj.show.indexerid, epObj.season, epObj.episode)
    # )

    # Search for release in snatch history
    results = failed_db_con.select(
        "SELECT release, provider, date FROM history WHERE showid=? AND season=? AND episode=?", [epObj.show.indexerid, epObj.season, epObj.episode]
    )

    for result in results:
        release = str(result["release"])
        provider = str(result["provider"])
        date = result["date"]

        # Clear any incomplete snatch records for this release if any exist
        failed_db_con.action("DELETE FROM history WHERE release=? AND date!=?", [release, date])

        # Found a previously failed release
        logger.debug("Failed release found for season ({0}): ({1})".format(epObj.season, result["release"]))
        return release, provider

    # Release was not found
    logger.debug("No releases found for season ({0}) of ({1})".format(epObj.season, epObj.show.indexerid))
    return release, provider



# Copyright: 2013 Paul Traylor
# These sources are released under the terms of the MIT license: see LICENSE

"""
The gntp.notifier module is provided as a simple way to send notifications
using GNTP

.. note::
	This class is intended to mostly mirror the older Python bindings such
	that you should be able to replace instances of the old bindings with
	this class.
	`Original Python bindings <http://code.google.com/p/growl/source/browse/Bindings/python/Growl.py>`_

"""
import logging
import platform
import socket
import sys

from gntp.version import __version__
import gntp.core
import gntp.errors as errors
import gntp.shim

__all__ = [
	'mini',
	'GrowlNotifier',
]

logger = logging.getLogger(__name__)


class GrowlNotifier(object):
	"""Helper class to simplfy sending Growl messages

	:param string applicationName: Sending application name
	:param list notification: List of valid notifications
	:param list defaultNotifications: List of notifications that should be enabled
		by default
	:param string applicationIcon: Icon URL
	:param string hostname: Remote host
	:param integer port: Remote port
	"""

	passwordHash = 'MD5'
	socketTimeout = 3

	def __init__(self, applicationName='Python GNTP', notifications=[],
			defaultNotifications=None, applicationIcon=None, hostname='localhost',
			password=None, port=23053):

		self.applicationName = applicationName
		self.notifications = list(notifications)
		if defaultNotifications:
			self.defaultNotifications = list(defaultNotifications)
		else:
			self.defaultNotifications = self.notifications
		self.applicationIcon = applicationIcon

		self.password = password
		self.hostname = hostname
		self.port = int(port)

	def _checkIcon(self, data):
		'''
		Check the icon to see if it's valid

		If it's a simple URL icon, then we return True. If it's a data icon
		then we return False
		'''
		logger.info('Checking icon')

		return gntp.shim.u(data)[:4] in ['http', 'file']

	def register(self):
		"""Send GNTP Registration

		.. warning::
			Before sending notifications to Growl, you need to have
			sent a registration message at least once
		"""
		logger.info('Sending registration to %s:%s', self.hostname, self.port)
		register = gntp.core.GNTPRegister()
		register.add_header('Application-Name', self.applicationName)
		for notification in self.notifications:
			enabled = notification in self.defaultNotifications
			register.add_notification(notification, enabled)
		if self.applicationIcon:
			if self._checkIcon(self.applicationIcon):
				register.add_header('Application-Icon', self.applicationIcon)
			else:
				resource = register.add_resource(self.applicationIcon)
				register.add_header('Application-Icon', resource)
		if self.password:
			register.set_password(self.password, self.passwordHash)
		self.add_origin_info(register)
		self.register_hook(register)
		return self._send('register', register)

	def notify(self, noteType, title, description, icon=None, sticky=False,
			priority=None, callback=None, identifier=None, custom={}):
		"""Send a GNTP notifications

		.. warning::
			Must have registered with growl beforehand or messages will be ignored

		:param string noteType: One of the notification names registered earlier
		:param string title: Notification title (usually displayed on the notification)
		:param string description: The main content of the notification
		:param string icon: Icon URL path
		:param boolean sticky: Sticky notification
		:param integer priority: Message priority level from -2 to 2
		:param string callback:  URL callback
		:param dict custom: Custom attributes. Key names should be prefixed with X-
			according to the spec but this is not enforced by this class

		.. warning::
			For now, only URL callbacks are supported. In the future, the
			callback argument will also support a function
		"""
		logger.info('Sending notification [%s] to %s:%s', noteType, self.hostname, self.port)
		assert noteType in self.notifications
		notice = gntp.core.GNTPNotice()
		notice.add_header('Application-Name', self.applicationName)
		notice.add_header('Notification-Name', noteType)
		notice.add_header('Notification-Title', title)
		if self.password:
			notice.set_password(self.password, self.passwordHash)
		if sticky:
			notice.add_header('Notification-Sticky', sticky)
		if priority:
			notice.add_header('Notification-Priority', priority)
		if icon:
			if self._checkIcon(icon):
				notice.add_header('Notification-Icon', icon)
			else:
				resource = notice.add_resource(icon)
				notice.add_header('Notification-Icon', resource)

		if description:
			notice.add_header('Notification-Text', description)
		if callback:
			notice.add_header('Notification-Callback-Target', callback)
		if identifier:
			notice.add_header('Notification-Coalescing-ID', identifier)

		for key in custom:
			notice.add_header(key, custom[key])

		self.add_origin_info(notice)
		self.notify_hook(notice)

		return self._send('notify', notice)

	def subscribe(self, id, name, port):
		"""Send a Subscribe request to a remote machine"""
		sub = gntp.core.GNTPSubscribe()
		sub.add_header('Subscriber-ID', id)
		sub.add_header('Subscriber-Name', name)
		sub.add_header('Subscriber-Port', port)
		if self.password:
			sub.set_password(self.password, self.passwordHash)

		self.add_origin_info(sub)
		self.subscribe_hook(sub)

		return self._send('subscribe', sub)

	def add_origin_info(self, packet):
		"""Add optional Origin headers to message"""
		packet.add_header('Origin-Machine-Name', platform.node())
		packet.add_header('Origin-Software-Name', 'gntp.py')
		packet.add_header('Origin-Software-Version', __version__)
		packet.add_header('Origin-Platform-Name', platform.system())
		packet.add_header('Origin-Platform-Version', platform.platform())

	def register_hook(self, packet):
		pass

	def notify_hook(self, packet):
		pass

	def subscribe_hook(self, packet):
		pass

	def _send(self, messagetype, packet):
		"""Send the GNTP Packet"""

		packet.validate()
		data = packet.encode()

		logger.debug('To : %s:%s <%s>\n%s', self.hostname, self.port, packet.__class__, data)

		s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		s.settimeout(self.socketTimeout)
		try:
			s.connect((self.hostname, self.port))
			s.send(data)
			recv_data = s.recv(1024)
			while not recv_data.endswith(gntp.shim.b("\r\n\r\n")):
				recv_data += s.recv(1024)
		except socket.error:
			# Python2.5 and Python3 compatibile exception
			exc = sys.exc_info()[1]
			raise errors.NetworkError(exc)

		response = gntp.core.parse_gntp(recv_data)
		s.close()

		logger.debug('From : %s:%s <%s>\n%s', self.hostname, self.port, response.__class__, response)

		if type(response) == gntp.core.GNTPOK:
			return True
		logger.error('Invalid response: %s', response.error())
		return response.error()


def mini(description, applicationName='PythonMini', noteType="Message",
			title="Mini Message", applicationIcon=None, hostname='localhost',
			password=None, port=23053, sticky=False, priority=None,
			callback=None, notificationIcon=None, identifier=None,
			notifierFactory=GrowlNotifier):
	"""Single notification function

	Simple notification function in one line. Has only one required parameter
	and attempts to use reasonable defaults for everything else
	:param string description: Notification message

	.. warning::
			For now, only URL callbacks are supported. In the future, the
			callback argument will also support a function
	"""
	try:
		growl = notifierFactory(
			applicationName=applicationName,
			notifications=[noteType],
			defaultNotifications=[noteType],
			applicationIcon=applicationIcon,
			hostname=hostname,
			password=password,
			port=port,
		)
		result = growl.register()
		if result is not True:
			return result

		return growl.notify(
			noteType=noteType,
			title=title,
			description=description,
			icon=notificationIcon,
			sticky=sticky,
			priority=priority,
			callback=callback,
			identifier=identifier,
		)
	except Exception:
		# We want the "mini" function to be simple and swallow Exceptions
		# in order to be less invasive
		logger.exception("Growl error")

if __name__ == '__main__':
	# If we're running this module directly we're likely running it as a test
	# so extra debugging is useful
	logging.basicConfig(level=logging.INFO)
	mini('Testing mini notification')



class ComparableMixin(object):
    def _compare(self, other, method):
        try:
            return method(self._cmpkey(), other._cmpkey())
        except (AttributeError, TypeError):
            # _cmpkey not implemented, or return different type,
            # so I can't compare with "other".
            return NotImplemented

    def __lt__(self, other):
        return self._compare(other, lambda s, o: s < o)

    def __le__(self, other):
        return self._compare(other, lambda s, o: s <= o)

    def __eq__(self, other):
        return self._compare(other, lambda s, o: s == o)

    def __ge__(self, other):
        return self._compare(other, lambda s, o: s >= o)

    def __gt__(self, other):
        return self._compare(other, lambda s, o: s > o)

    def __ne__(self, other):
        return self._compare(other, lambda s, o: s != o)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: generated by @lolobosse script
'''
LOCALE = [
    ["éppen most", "éppen most"],
    ["%s másodperce", "%s másodpercen belül"],
    ["1 perce", "1 percen belül"],
    ["%s perce", "%s percen belül"],
    ["1 órája", "1 órán belül"],
    ["%s órája", "%s órán belül"],
    ["1 napja", "1 napon belül"],
    ["%s napja", "%s napon belül"],
    ["1 hete", "1 héten belül"],
    ["%s hete", "%s héten belül"],
    ["1 hónapja", "1 hónapon belül"],
    ["%s hónapja", "%s hónapon belül"],
    ["1 éve", "1 éven belül"],
    ["%s éve", "%s éven belül"]
]
        


# -*- coding: utf-8 -*-
import json
import logging
import os

from babelfish import Language, language_converters
from requests import Session

from . import Provider
from ..subtitle import Subtitle, fix_line_ending

logger = logging.getLogger(__name__)

language_converters.register('shooter = subliminal.converters.shooter:ShooterConverter')


class ShooterSubtitle(Subtitle):
    """Shooter Subtitle."""
    provider_name = 'shooter'

    def __init__(self, language, hash, download_link):
        super(ShooterSubtitle, self).__init__(language)
        self.hash = hash
        self.download_link = download_link

    @property
    def id(self):
        return self.download_link

    @property
    def info(self):
        return self.hash

    def get_matches(self, video):
        matches = set()

        # hash
        if 'shooter' in video.hashes and video.hashes['shooter'] == self.hash:
            matches.add('hash')

        return matches


class ShooterProvider(Provider):
    """Shooter Provider."""
    languages = {Language(l) for l in ['eng', 'zho']}
    server_url = 'https://www.shooter.cn/api/subapi.php'
    subtitle_class = ShooterSubtitle

    def __init__(self):
        self.session = None

    def initialize(self):
        self.session = Session()
        self.session.headers['User-Agent'] = self.user_agent

    def terminate(self):
        self.session.close()

    def query(self, language, filename, hash=None):
        # query the server
        params = {'filehash': hash, 'pathinfo': os.path.realpath(filename), 'format': 'json', 'lang': language.shooter}
        logger.debug('Searching subtitles %r', params)
        r = self.session.post(self.server_url, params=params, timeout=10)
        r.raise_for_status()

        # handle subtitles not found
        if r.content == b'\xff':
            logger.debug('No subtitles found')
            return []

        # parse the subtitles
        results = json.loads(r.text)
        subtitles = [self.subtitle_class(language, hash, t['Link']) for s in results for t in s['Files']]

        return subtitles

    def list_subtitles(self, video, languages):
        return [s for l in languages for s in self.query(l, video.name, video.hashes.get('shooter'))]

    def download_subtitle(self, subtitle):
        logger.info('Downloading subtitle %r', subtitle)
        r = self.session.get(subtitle.download_link, timeout=10)
        r.raise_for_status()

        subtitle.content = fix_line_ending(r.content)



from __future__ import absolute_import

import ast
import re
import operator as op
import pyparsing

from ..exceptions import CloudflareSolveError
from . import JavaScriptInterpreter

# ------------------------------------------------------------------------------- #

_OP_MAP = {
    ast.Add: op.add,
    ast.Sub: op.sub,
    ast.Mult: op.mul,
    ast.Div: op.truediv,
    ast.Invert: op.neg,
}

# ------------------------------------------------------------------------------- #


class Calc(ast.NodeVisitor):

    def visit_BinOp(self, node):
        return _OP_MAP[type(node.op)](self.visit(node.left), self.visit(node.right))

    # ------------------------------------------------------------------------------- #

    def visit_Num(self, node):
        return node.n

    # ------------------------------------------------------------------------------- #

    def visit_Expr(self, node):
        return self.visit(node.value)

    # ------------------------------------------------------------------------------- #

    @classmethod
    def doMath(cls, expression):
        tree = ast.parse(expression)
        calc = cls()
        return calc.visit(tree.body[0])

# ------------------------------------------------------------------------------- #


class Parentheses(object):

    def fix(self, s):
        res = []
        self.visited = set([s])
        self.dfs(s, self.invalid(s), res)
        return res

    # ------------------------------------------------------------------------------- #

    def dfs(self, s, n, res):
        if n == 0:
            res.append(s)
            return
        for i in range(len(s)):
            if s[i] in ['(', ')']:
                s_new = s[:i] + s[i + 1:]
                if s_new not in self.visited and self.invalid(s_new) < n:
                    self.visited.add(s_new)
                    self.dfs(s_new, self.invalid(s_new), res)

    # ------------------------------------------------------------------------------- #

    def invalid(self, s):
        plus = minus = 0
        memo = {"(": 1, ")": -1}
        for c in s:
            plus += memo.get(c, 0)
            minus += 1 if plus < 0 else 0
            plus = max(0, plus)
        return plus + minus

# ------------------------------------------------------------------------------- #


class ChallengeInterpreter(JavaScriptInterpreter):

    def __init__(self):
        super(ChallengeInterpreter, self).__init__('native')

    # ------------------------------------------------------------------------------- #

    def eval(self, body, domain):

        operators = {
            '+': op.add,
            '-': op.sub,
            '*': op.mul,
            '/': op.truediv
        }

        # ------------------------------------------------------------------------------- #

        def flatten(lists):
            return sum(map(flatten, lists), []) if isinstance(lists, list) else [lists]

        # ------------------------------------------------------------------------------- #

        def jsfuckToNumber(jsFuck):
            # "Clean Up" JSFuck
            jsFuck = jsFuck.replace('!+[]', '1').replace('!![]', '1').replace('[]', '0')
            jsFuck = jsFuck.lstrip('+').replace('(+', '(').replace(' ', '')
            jsFuck = Parentheses().fix(jsFuck)[0]

            # Hackery Parser for Math
            stack = []
            bstack = []

            for i in flatten(pyparsing.nestedExpr().parseString(jsFuck).asList()):
                if i == '+':
                    stack.append(bstack)
                    bstack = []
                    continue
                bstack.append(i)
            stack.append(bstack)

            return int(''.join([str(Calc.doMath(''.join(i))) for i in stack]))

        # ------------------------------------------------------------------------------- #

        def divisorMath(payload, needle, domain):
            jsfuckMath = payload.split('/')
            if needle in jsfuckMath[1]:
                expression = re.findall(r"^(.*?)(.)\(function", jsfuckMath[1])[0]

                expression_value = operators[expression[1]](
                    float(jsfuckToNumber(expression[0])),
                    float(ord(domain[jsfuckToNumber(jsfuckMath[1][
                        jsfuckMath[1].find('"("+p+")")}') + len('"("+p+")")}'):-2
                    ])]))
                )
            else:
                expression_value = jsfuckToNumber(jsfuckMath[1])

            expression_value = jsfuckToNumber(jsfuckMath[0]) / float(expression_value)

            return expression_value

        # ------------------------------------------------------------------------------- #

        def challengeSolve(body, domain):
            jschl_answer = 0

            try:
                jsfuckChallenge = re.search(
                    r"setTimeout\(function\(\){\s+var.*?f,\s*(?P<variable>\w+).*?:(?P<init>\S+)};"
                    r".*?\('challenge-form'\);.*?;(?P<challenge>.*?a\.value)\s*=\s*\S+\.toFixed\(10\);",
                    body,
                    re.DOTALL | re.MULTILINE
                ).groupdict()
            except AttributeError:
                raise CloudflareSolveError('There was an issue extracting "jsfuckChallenge" from the Cloudflare challenge.')

            kJSFUCK = re.search(r'(;|)\s*k.=(?P<kJSFUCK>\S+);', jsfuckChallenge['challenge'], re.S | re.M)
            if kJSFUCK:
                try:
                    kJSFUCK = jsfuckToNumber(kJSFUCK.group('kJSFUCK'))
                except IndexError:
                    raise CloudflareSolveError('There was an issue extracting "kJSFUCK" from the Cloudflare challenge.')

                try:
                    kID = re.search(r"\s*k\s*=\s*'(?P<kID>\S+)';", body).group('kID')
                except IndexError:
                    raise CloudflareSolveError('There was an issue extracting "kID" from the Cloudflare challenge.')

                try:
                    r = re.compile(r'<div id="{}(?P<id>\d+)">\s*(?P<jsfuck>[^<>]*)</div>'.format(kID))

                    kValues = {}
                    for m in r.finditer(body):
                        kValues[int(m.group('id'))] = m.group('jsfuck')

                    jsfuckChallenge['k'] = kValues[kJSFUCK]
                except (AttributeError, IndexError):
                    raise CloudflareSolveError('There was an issue extracting "kValues" from the Cloudflare challenge.')

            jsfuckChallenge['challenge'] = re.finditer(
                r'{}.*?([+\-*/])=(.*?);(?=a\.value|{})'.format(
                    jsfuckChallenge['variable'],
                    jsfuckChallenge['variable']
                ),
                jsfuckChallenge['challenge']
            )

            # ------------------------------------------------------------------------------- #

            if '/' in jsfuckChallenge['init']:
                val = jsfuckChallenge['init'].split('/')
                jschl_answer = jsfuckToNumber(val[0]) / float(jsfuckToNumber(val[1]))
            else:
                jschl_answer = jsfuckToNumber(jsfuckChallenge['init'])

            # ------------------------------------------------------------------------------- #

            for expressionMatch in jsfuckChallenge['challenge']:
                oper, expression = expressionMatch.groups()

                if '/' in expression:
                    expression_value = divisorMath(expression, 'function(p)', domain)
                else:
                    if 'Element' in expression:
                        expression_value = divisorMath(jsfuckChallenge['k'], '"("+p+")")}', domain)
                    else:
                        expression_value = jsfuckToNumber(expression)

                jschl_answer = operators[oper](jschl_answer, expression_value)

            # ------------------------------------------------------------------------------- #

            # if not jsfuckChallenge['k'] and '+ t.length' in body:
            #    jschl_answer += len(domain)

            # ------------------------------------------------------------------------------- #

            return '{0:.10f}'.format(jschl_answer)

        # ------------------------------------------------------------------------------- #

        return challengeSolve(body, domain)

# ------------------------------------------------------------------------------- #


ChallengeInterpreter()



# -*- coding: utf-8 -*-

"""
This module implements the Search functionality of TheTVDb API.
Allows to search for tv series in the db.

See [Search API section](https://api.thetvdb.com/swagger#!/Search)
"""

from .base import TVDB

class Search(TVDB):
    """
    Class that allow to search for series with filters.
    """
    _BASE_PATH = 'search'
    _URLS = {
        'series': '/series',
        'series_params': '/series/params'
    }

    def series(self, name='', imdbId='', zap2itId='', language=''):
        """
        Search series with the information provided.

        You can set `name` to search for a series with that name. You can set `imdbId` 
        to search a series with the provided imdb id. You can set `zap2itId` 
        to search a series with the provided zap2it id. You can set `language` to 
        retrieve the results with the provided language.

        Returns a list series with basic information that matches your search.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> search = tvdb.Search()
            >>> reponse = search.series("doctor who 2005")
            >>> search.series[0]['seriesName']
            'Doctor Who (2005)'

        """
        path = self._get_path('series')

        filters = {}
        if name:
            filters['name'] = name
        if imdbId:
            filters['imdbId'] = imdbId
        if zap2itId:
            filters['zap2itId'] = zap2itId

        self._set_language(language)
        response = self._GET(path, params=filters)
        self._set_attrs_to_values({'series': response})
        return response

    def series_params(self):
        """
        Return the available search params.
        
        Returns:
            A dict respresentation of the JSON returned from the API.
        """
        path = self._get_path('series_params')

        response = self._GET(path)
        self._set_attrs_to_values({'series_params': response})
        return response



import re

from twilio.rest import Client, TwilioException

from sickchill import logger, settings
from sickchill.oldbeard import common


class Notifier(object):

    number_regex = re.compile(r"^\+1-\d{3}-\d{3}-\d{4}$")
    account_regex = re.compile(r"^AC[a-z0-9]{32}$")
    auth_regex = re.compile(r"^[a-z0-9]{32}$")
    phone_regex = re.compile(r"^PN[a-z0-9]{32}$")

    def notify_snatch(self, ep_name):
        if settings.TWILIO_NOTIFY_ONSNATCH:
            self._notifyTwilio(common.notifyStrings[common.NOTIFY_SNATCH] + ": " + ep_name)

    def notify_download(self, ep_name):
        if settings.TWILIO_NOTIFY_ONDOWNLOAD:
            self._notifyTwilio(common.notifyStrings[common.NOTIFY_DOWNLOAD] + ": " + ep_name)

    def notify_subtitle_download(self, ep_name, lang):
        if settings.TWILIO_NOTIFY_ONSUBTITLEDOWNLOAD:
            self._notifyTwilio(common.notifyStrings[common.NOTIFY_SUBTITLE_DOWNLOAD] + " " + ep_name + ": " + lang)

    def notify_git_update(self, new_version):
        if settings.USE_TWILIO:
            update_text = common.notifyStrings[common.NOTIFY_GIT_UPDATE_TEXT]
            self._notifyTwilio(update_text + new_version)

    def notify_login(self, ipaddress=""):
        if settings.USE_TWILIO:
            update_text = common.notifyStrings[common.NOTIFY_LOGIN_TEXT]
            title = common.notifyStrings[common.NOTIFY_LOGIN]
            self._notifyTwilio(title + " - " + update_text.format(ipaddress))

    def test_notify(self):
        try:
            if not self.number.capabilities["sms"]:
                return False

            return self._notifyTwilio(_("This is a test notification from SickChill"), force=True, allow_raise=True)
        except TwilioException:
            return False

    @property
    def number(self):
        return self.client.api.incoming_phone_numbers.get(settings.TWILIO_PHONE_SID).fetch()

    @property
    def client(self):
        return Client(settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN)

    def _notifyTwilio(self, message="", force=False, allow_raise=False):
        if not (settings.USE_TWILIO or force or self.number_regex.match(settings.TWILIO_TO_NUMBER)):
            return False

        logger.debug("Sending Twilio SMS: " + message)

        try:
            self.client.messages.create(
                body=message,
                to=settings.TWILIO_TO_NUMBER,
                from_=self.number.phone_number,
            )
        except TwilioException as e:
            logger.exception("Twilio notification failed:" + str(e))

            if allow_raise:
                raise e

        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: generated by @lolobosse script
'''
LOCALE = [
    ["recent", "binnenkort"],
    ["%s seconden geleden", "binnen %s seconden"],
    ["1 minuut geleden", "binnen 1 minuut"],
    ["%s minuten geleden", "binnen %s minuten"],
    ["1 uur geleden", "binnen 1 uur"],
    ["%s uren geleden", "binnen %s uren"],
    ["1 dag geleden", "binnen 1 dag"],
    ["%s dagen geleden", "binnen %s dagen"],
    ["1 week geleden", "binnen 1 week"],
    ["%s weken geleden", "binnen %s weken"],
    ["1 maand geleden", "binnen 1 maand"],
    ["%s maanden geleden", "binnen %s maanden"],
    ["1 jaar geleden", "binnen 1 jaar"],
    ["%s jaren geleden", "binnen %s jaren"]
]
        


# -*- coding: utf-8 -*-
import io
import logging
import re

from babelfish import Language, language_converters
from guessit import guessit
try:
    from lxml import etree
except ImportError:
    try:
        import xml.etree.cElementTree as etree
    except ImportError:
        import xml.etree.ElementTree as etree
from requests import Session
from zipfile import ZipFile

from . import Provider
from ..exceptions import ProviderError
from ..matches import guess_matches
from ..subtitle import Subtitle, fix_line_ending
from ..video import Episode

logger = logging.getLogger(__name__)


class PodnapisiSubtitle(Subtitle):
    """Podnapisi Subtitle."""
    provider_name = 'podnapisi'

    def __init__(self, language, hearing_impaired, page_link, pid, releases, title, season=None, episode=None,
                 year=None):
        super(PodnapisiSubtitle, self).__init__(language, hearing_impaired=hearing_impaired, page_link=page_link)
        self.pid = pid
        self.releases = releases
        self.title = title
        self.season = season
        self.episode = episode
        self.year = year

    @property
    def id(self):
        return self.pid

    @property
    def info(self):
        return ' '.join(self.releases) or self.pid

    def get_matches(self, video):
        matches = guess_matches(video, {
            'title': self.title,
            'year': self.year,
            'season': self.season,
            'episode': self.episode
        })

        video_type = 'episode' if isinstance(video, Episode) else 'movie'
        for release in self.releases:
            matches |= guess_matches(video, guessit(release, {'type': video_type}))

        return matches


class PodnapisiProvider(Provider):
    """Podnapisi Provider."""
    languages = ({Language('por', 'BR'), Language('srp', script='Latn')} |
                 {Language.fromalpha2(l) for l in language_converters['alpha2'].codes})
    server_url = 'https://www.podnapisi.net/subtitles/'
    subtitle_class = PodnapisiSubtitle

    def __init__(self):
        self.session = None

    def initialize(self):
        self.session = Session()
        self.session.headers['User-Agent'] = self.user_agent

    def terminate(self):
        self.session.close()

    def query(self, language, keyword, season=None, episode=None, year=None):
        # set parameters, see http://www.podnapisi.net/forum/viewtopic.php?f=62&t=26164#p212652
        params = {'sXML': 1, 'sL': str(language), 'sK': keyword}
        is_episode = False
        if season and episode:
            is_episode = True
            params['sTS'] = season
            params['sTE'] = episode
        if year:
            params['sY'] = year

        # loop over paginated results
        logger.info('Searching subtitles %r', params)
        subtitles = []
        pids = set()
        while True:
            # query the server
            r = self.session.get(self.server_url + 'search/old', params=params, timeout=10)
            r.raise_for_status()
            xml = etree.fromstring(r.content)

            # exit if no results
            if not int(xml.find('pagination/results').text):
                logger.debug('No subtitles found')
                break

            # loop over subtitles
            for subtitle_xml in xml.findall('subtitle'):
                # read xml elements
                pid = subtitle_xml.find('pid').text
                # ignore duplicates, see http://www.podnapisi.net/forum/viewtopic.php?f=62&t=26164&start=10#p213321
                if pid in pids:
                    continue

                language = Language.fromietf(subtitle_xml.find('language').text)
                hearing_impaired = 'n' in (subtitle_xml.find('flags').text or '')
                page_link = subtitle_xml.find('url').text
                releases = []
                if subtitle_xml.find('release').text:
                    for release in subtitle_xml.find('release').text.split():
                        release = re.sub(r'\.+$', '', release)  # remove trailing dots
                        release = ''.join(filter(lambda x: ord(x) < 128, release))  # remove non-ascii characters
                        releases.append(release)
                title = subtitle_xml.find('title').text
                season = int(subtitle_xml.find('tvSeason').text)
                episode = int(subtitle_xml.find('tvEpisode').text)
                year = int(subtitle_xml.find('year').text)

                if is_episode:
                    subtitle = self.subtitle_class(language, hearing_impaired, page_link, pid, releases, title,
                                                   season=season, episode=episode, year=year)
                else:
                    subtitle = self.subtitle_class(language, hearing_impaired, page_link, pid, releases, title,
                                                   year=year)

                logger.debug('Found subtitle %r', subtitle)
                subtitles.append(subtitle)
                pids.add(pid)

            # stop on last page
            if int(xml.find('pagination/current').text) >= int(xml.find('pagination/count').text):
                break

            # increment current page
            params['page'] = int(xml.find('pagination/current').text) + 1
            logger.debug('Getting page %d', params['page'])

        return subtitles

    def list_subtitles(self, video, languages):
        season = episode = None
        if isinstance(video, Episode):
            titles = [video.series] + video.alternative_series
            season = video.season
            episode = video.episode
        else:
            titles = [video.title] + video.alternative_titles

        for title in titles:
            subtitles = [s for l in languages for s in
                         self.query(l, title, season=season, episode=episode, year=video.year)]
            if subtitles:
                return subtitles

        return []

    def download_subtitle(self, subtitle):
        # download as a zip
        logger.info('Downloading subtitle %r', subtitle)
        r = self.session.get(self.server_url + subtitle.pid + '/download', params={'container': 'zip'}, timeout=10)
        r.raise_for_status()

        # open the zip
        with ZipFile(io.BytesIO(r.content)) as zf:
            if len(zf.namelist()) > 1:
                raise ProviderError('More than one file to unzip')

            subtitle.content = fix_line_ending(zf.read(zf.namelist()[0]))



"""
Importing certain classes into the global namespace.
"""

from beekeeper.api import API
from beekeeper.hive import Hive
from beekeeper.data_handlers import DataHandler
from beekeeper.variable_handlers import VariableHandler, set_content_type
from beekeeper.variable_handlers import render as render_variables



__author__ = 'Andrea De Marco <24erre@gmail.com>'
__maintainer__ = 'Pol Canelles <canellestudi@gmail.com>'
__version__ = '2.0.0'
__classifiers__ = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Topic :: Internet :: WWW/HTTP',
    'Topic :: Software Development :: Libraries',
]
__copyright__ = "2012, %s " % __author__
__license__ = """
   Copyright %s.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either expressed or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
""" % __copyright__

__docformat__ = 'restructuredtext en'

__doc__ = """
:abstract: Python interface to fanart.tv API (v3)
:version: %s
:author: %s
:contact: http://z4r.github.com/
:date: 2012-04-04
:copyright: %s
""" % (__version__, __author__, __license__)


def values(obj):
    return [v for k, v in obj.__dict__.items() if not k.startswith('_')]


BASEURL = 'http://webservice.fanart.tv/v3'


class FORMAT(object):
    JSON = 'JSON'
    XML = 'XML'
    PHP = 'PHP'


class WS(object):
    MUSIC = 'music'
    MOVIE = 'movies'
    TV = 'tv'


class TYPE(object):
    ALL = 'all'

    class TV(object):
        ART = 'clearart'
        LOGO = 'clearlogo'
        CHARACTER = 'characterart'
        THUMB = 'tvthumb'
        SEASONTHUMB = 'seasonthumb'
        SEASONBANNER = 'seasonbanner'
        SEASONPOSTER = 'seasonposter'
        BACKGROUND = 'showbackground'
        HDLOGO = 'hdtvlogo'
        HDART = 'hdclearart'
        POSTER = 'tvposter'
        BANNER = 'tvbanner'

    class MUSIC(object):
        DISC = 'cdart'
        LOGO = 'musiclogo'
        BACKGROUND = 'artistbackground'
        COVER = 'albumcover'
        THUMB = 'artistthumb'

    class MOVIE(object):
        ART = 'movieart'
        LOGO = 'movielogo'
        DISC = 'moviedisc'
        POSTER = 'movieposter'
        BACKGROUND = 'moviebackground'
        HDLOGO = 'hdmovielogo'
        HDART = 'hdmovieclearart'
        BANNER = 'moviebanner'
        THUMB = 'moviethumb'


class SORT(object):
    POPULAR = 1
    NEWEST = 2
    OLDEST = 3


class LIMIT(object):
    ONE = 1
    ALL = 2


FORMAT_LIST = values(FORMAT)
WS_LIST = values(WS)
TYPE_LIST = values(TYPE.MUSIC) + values(TYPE.TV) + values(TYPE.MOVIE) + [TYPE.ALL]
MUSIC_TYPE_LIST = values(TYPE.MUSIC) + [TYPE.ALL]
TV_TYPE_LIST = values(TYPE.TV) + [TYPE.ALL]
MOVIE_TYPE_LIST = values(TYPE.MOVIE) + [TYPE.ALL]
SORT_LIST = values(SORT)
LIMIT_LIST = values(LIMIT)



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta
from functools import wraps
import logging
import re

from babelfish import Country
import guessit
import requests

from .. import __short_version__
from ..cache import REFINER_EXPIRATION_TIME, region
from ..utils import sanitize
from ..video import Episode

logger = logging.getLogger(__name__)

series_re = re.compile(r'^(?P<series>.*?)(?: \((?:(?P<year>\d{4})|(?P<country>[A-Z]{2}))\))?$')


def requires_auth(func):
    """Decorator for :class:`TVDBClient` methods that require authentication"""
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        if self.token is None or self.token_expired:
            self.login()
        elif self.token_needs_refresh:
            self.refresh_token()
        return func(self, *args, **kwargs)
    return wrapper


class TVDBClient(object):
    """TVDB REST API Client

    :param str apikey: API key to use.
    :param str username: username to use.
    :param str password: password to use.
    :param str language: language of the responses.
    :param session: session object to use.
    :type session: :class:`requests.sessions.Session` or compatible.
    :param dict headers: additional headers.
    :param int timeout: timeout for the requests.

    """
    #: Base URL of the API
    base_url = 'https://api.thetvdb.com'

    #: Token lifespan
    token_lifespan = timedelta(hours=1)

    #: Minimum token age before a :meth:`refresh_token` is triggered
    refresh_token_every = timedelta(minutes=30)

    def __init__(self, apikey=None, username=None, password=None, language='en', session=None, headers=None,
                 timeout=10):
        #: API key
        self.apikey = apikey

        #: Username
        self.username = username

        #: Password
        self.password = password

        #: Last token acquisition date
        self.token_date = datetime.utcnow() - self.token_lifespan

        #: Session for the requests
        self.session = session or requests.Session()
        self.session.timeout = timeout
        self.session.headers.update(headers or {})
        self.session.headers['Content-Type'] = 'application/json'
        self.session.headers['Accept-Language'] = language

    @property
    def language(self):
        return self.session.headers['Accept-Language']

    @language.setter
    def language(self, value):
        self.session.headers['Accept-Language'] = value

    @property
    def token(self):
        if 'Authorization' not in self.session.headers:
            return None
        return self.session.headers['Authorization'][7:]

    @property
    def token_expired(self):
        return datetime.utcnow() - self.token_date > self.token_lifespan

    @property
    def token_needs_refresh(self):
        return datetime.utcnow() - self.token_date > self.refresh_token_every

    def login(self):
        """Login"""
        # perform the request
        data = {'apikey': self.apikey, 'username': self.username, 'password': self.password}
        r = self.session.post(self.base_url + '/login', json=data)
        r.raise_for_status()

        # set the Authorization header
        self.session.headers['Authorization'] = 'Bearer ' + r.json()['token']

        # update token_date
        self.token_date = datetime.utcnow()

    def refresh_token(self):
        """Refresh token"""
        # perform the request
        r = self.session.get(self.base_url + '/refresh_token')
        r.raise_for_status()

        # set the Authorization header
        self.session.headers['Authorization'] = 'Bearer ' + r.json()['token']

        # update token_date
        self.token_date = datetime.utcnow()

    @requires_auth
    def search_series(self, name=None, imdb_id=None, zap2it_id=None):
        """Search series"""
        # perform the request
        params = {'name': name, 'imdbId': imdb_id, 'zap2itId': zap2it_id}
        r = self.session.get(self.base_url + '/search/series', params=params)
        if r.status_code == 404:
            return None
        r.raise_for_status()

        return r.json()['data']

    @requires_auth
    def get_series(self, id):
        """Get series"""
        # perform the request
        r = self.session.get(self.base_url + '/series/{}'.format(id))
        if r.status_code == 404:
            return None
        r.raise_for_status()

        return r.json()['data']

    @requires_auth
    def get_series_actors(self, id):
        """Get series actors"""
        # perform the request
        r = self.session.get(self.base_url + '/series/{}/actors'.format(id))
        if r.status_code == 404:
            return None
        r.raise_for_status()

        return r.json()['data']

    @requires_auth
    def get_series_episodes(self, id, page=1):
        """Get series episodes"""
        # perform the request
        params = {'page': page}
        r = self.session.get(self.base_url + '/series/{}/episodes'.format(id), params=params)
        if r.status_code == 404:
            return None
        r.raise_for_status()

        return r.json()

    @requires_auth
    def query_series_episodes(self, id, absolute_number=None, aired_season=None, aired_episode=None, dvd_season=None,
                              dvd_episode=None, imdb_id=None, page=1):
        """Query series episodes"""
        # perform the request
        params = {'absoluteNumber': absolute_number, 'airedSeason': aired_season, 'airedEpisode': aired_episode,
                  'dvdSeason': dvd_season, 'dvdEpisode': dvd_episode, 'imdbId': imdb_id, 'page': page}
        r = self.session.get(self.base_url + '/series/{}/episodes/query'.format(id), params=params)
        if r.status_code == 404:
            return None
        r.raise_for_status()

        return r.json()

    @requires_auth
    def get_episode(self, id):
        """Get episode"""
        # perform the request
        r = self.session.get(self.base_url + '/episodes/{}'.format(id))
        if r.status_code == 404:
            return None
        r.raise_for_status()

        return r.json()['data']


#: User-Agent to use
user_agent = 'Subliminal/%s' % __short_version__

#: Configured instance of :class:`TVDBClient`
tvdb_client = TVDBClient('5EC930FB90DA1ADA', headers={'User-Agent': user_agent})

#: Configure guessit in order to use GuessitCountryConverter
guessit.api.configure()


@region.cache_on_arguments(expiration_time=REFINER_EXPIRATION_TIME)
def search_series(name):
    """Search series.

    :param str name: name of the series.
    :return: the search results.
    :rtype: list

    """
    return tvdb_client.search_series(name)


@region.cache_on_arguments(expiration_time=REFINER_EXPIRATION_TIME)
def get_series(id):
    """Get series.

    :param int id: id of the series.
    :return: the series data.
    :rtype: dict

    """
    return tvdb_client.get_series(id)


@region.cache_on_arguments(expiration_time=REFINER_EXPIRATION_TIME)
def get_series_episode(series_id, season, episode):
    """Get an episode of a series.

    :param int series_id: id of the series.
    :param int season: season number of the episode.
    :param int episode: episode number of the episode.
    :return: the episode data.
    :rtype: dict

    """
    result = tvdb_client.query_series_episodes(series_id, aired_season=season, aired_episode=episode)
    if result:
        return tvdb_client.get_episode(result['data'][0]['id'])


def refine(video, **kwargs):
    """Refine a video by searching `TheTVDB <http://thetvdb.com/>`_.

    .. note::

        This refiner only work for instances of :class:`~subliminal.video.Episode`.

    Several attributes can be found:

      * :attr:`~subliminal.video.Episode.series`
      * :attr:`~subliminal.video.Episode.year`
      * :attr:`~subliminal.video.Episode.series_imdb_id`
      * :attr:`~subliminal.video.Episode.series_tvdb_id`
      * :attr:`~subliminal.video.Episode.title`
      * :attr:`~subliminal.video.Video.imdb_id`
      * :attr:`~subliminal.video.Episode.tvdb_id`

    """
    # only deal with Episode videos
    if not isinstance(video, Episode):
        logger.error('Cannot refine episodes')
        return

    # exit if the information is complete
    if video.series_tvdb_id and video.tvdb_id:
        logger.debug('No need to search')
        return

    # search the series
    logger.info('Searching series %r', video.series)
    results = search_series(video.series.lower())
    if not results:
        logger.warning('No results for series')
        return
    logger.debug('Found %d results', len(results))

    # search for exact matches
    matching_results = []
    for result in results:
        matching_result = {}

        # use seriesName and aliases
        series_names = [result['seriesName']]
        series_names.extend(result['aliases'])

        # parse the original series as series + year or country
        original_match = series_re.match(result['seriesName']).groupdict()

        # parse series year
        series_year = None
        if result['firstAired']:
            series_year = datetime.strptime(result['firstAired'], '%Y-%m-%d').year

        # discard mismatches on year
        if video.year and series_year and video.year != series_year:
            logger.debug('Discarding series %r mismatch on year %d', result['seriesName'], series_year)
            continue

        # iterate over series names
        for series_name in series_names:
            # parse as series, year and country
            series, year, country = series_re.match(series_name).groups()
            if year:
                year = int(year)

            if country:
                country = Country.fromguessit(country)

            # discard mismatches on year
            if year and (video.original_series or video.year != year):
                logger.debug('Discarding series name %r mismatch on year %d', series, year)
                continue

            # discard mismatches on country
            if video.country and video.country != country:
                logger.debug('Discarding series name %r mismatch on country %r', series, country)
                continue

            # match on sanitized series name
            if sanitize(series) == sanitize(video.series):
                logger.debug('Found exact match on series %r', series_name)
                matching_result['match'] = {
                    'series': original_match['series'],
                    'year': series_year or year,
                    'country': country,
                    'original_series': original_match['year'] is None and country is None
                }
                break

        # add the result on match
        if matching_result:
            matching_result['data'] = result
            matching_results.append(matching_result)

    # exit if we don't have exactly 1 matching result
    if not matching_results:
        logger.error('No matching series found')
        return
    if len(matching_results) > 1:
        logger.error('Multiple matches found')
        return

    # get the series
    matching_result = matching_results[0]
    series = get_series(matching_result['data']['id'])

    # add series information
    logger.debug('Found series %r', series)
    video.series = matching_result['match']['series']
    video.alternative_series.extend(series['aliases'])
    video.year = matching_result['match']['year']
    video.country = matching_result['match']['country']
    video.original_series = matching_result['match']['original_series']
    video.series_tvdb_id = series['id']
    video.series_imdb_id = series['imdbId'] or None

    # get the episode
    logger.info('Getting series episode %dx%d', video.season, video.episode)
    episode = get_series_episode(video.series_tvdb_id, video.season, video.episode)
    if not episode:
        logger.warning('No results for episode')
        return

    # add episode information
    logger.debug('Found episode %r', episode)
    video.tvdb_id = episode['id']
    video.title = episode['episodeName'] or None
    video.imdb_id = episode['imdbId'] or None
